/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.annotation.processor;

import java.lang.reflect.Method;

import org.apache.commons.lang.StringUtils;
import org.loom.action.Action;
import org.loom.exception.ConfigException;
import org.loom.interceptor.MethodInterceptor;
import org.loom.interceptor.MethodInterceptor.AfterCreate;
import org.loom.interceptor.MethodInterceptor.BeforeExecute;
import org.loom.interceptor.MethodInterceptor.BeforePopulate;
import org.loom.interceptor.MethodInterceptor.BeforeValidate;
import org.loom.interceptor.MethodInterceptor.DoFinally;
import org.loom.interceptor.MethodInterceptor.OnError;
import org.loom.mapping.ActionMapping;
import org.loom.mapping.Event;
import org.loom.resolution.Resolution;
import org.loom.util.ClassUtils;


/**
 * Processes the {@link org.loom.annotation.Event} annotation
 * @author icoloma
 *
 */
public class EventAnnotationProcessor extends AbstractAnnotationProcessor {

	@Override
	public void process(Event event) {
		org.loom.annotation.Event annotation = event.getJavaMethod().getAnnotation(org.loom.annotation.Event.class);
		if (annotation != null) {
			
			addHandler(event, AfterCreate.class, annotation.afterCreate());
			addHandler(event, BeforePopulate.class, annotation.beforePopulate());
			addHandler(event, BeforeValidate.class, annotation.beforeValidate());
			addHandler(event, BeforeExecute.class, annotation.beforeExecute());
			addHandler(event, OnError.class, annotation.onError());
			addHandler(event, DoFinally.class, annotation.doFinally(), Exception.class);
			
			// default event
			if (annotation.defaultEvent()) {
				Event oldDefault = event.getActionMapping().getDefaultEvent();
				if (oldDefault != null && oldDefault != event) { // there are two different default events 
					throw new ConfigException("Two events are annotated as \"default\": " + oldDefault.getPrintName() + " and " + event.getPrintName());
				}
				event.getActionMapping().setDefaultEvent(event);
			}
			
		}
	}
	
	protected void addHandler(Event event, Class<? extends MethodInterceptor> interceptorClass, String[] methodNames, Class<?>... types) {
		ActionMapping mapping = event.getActionMapping();
		Class<? extends Action> actionClass = mapping.getActionClass();
		for (int i = 0; i < methodNames.length; i++) {
			String methodName = methodNames[i];
			
			if (StringUtils.isEmpty(methodName)) {
				return;
			}
			
			Method method = ClassUtils.getMethod(actionClass, methodName, types);
			if (method == null) {
				throw new ConfigException("Handler method " + methodName + " in action " + actionClass.getName() + " not found");
			}
			Class<?> returnType = method.getReturnType();
			if (!Resolution.class.isAssignableFrom(returnType) && returnType != Void.TYPE) {
				throw new ConfigException("Handler method " + methodName + " in action " + actionClass.getName() + " should return Resolution or void");
			}
			event.getInterceptors().addListener(MethodInterceptor.getInstance(interceptorClass, method));
			
			// remove as event handler
			mapping.removeEvent(methodName);
		}
		
	}

}
