/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.annotation.processor;

import java.lang.reflect.Method;
import java.util.List;
import java.util.Set;

import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.HEAD;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.QueryParam;

import org.loom.action.Action;
import org.loom.exception.ConfigException;
import org.loom.log.Log;
import org.loom.mapping.ActionMapping;
import org.loom.mapping.DefaultEventParameter;
import org.loom.mapping.Event;
import org.loom.mapping.EventParameter;
import org.loom.mapping.RequestParametersParameter;
import org.loom.mapping.UriParser;
import org.loom.mapping.UriToken;
import org.loom.servlet.HttpMethod;
import org.loom.util.StringUtils;

import com.google.common.collect.Sets;


/**
 * Processes JSR311 JAX-RS annotations
 * {@link Path}, {@link GET}, {@link POST}, {@link DELETE}, {@link PUT} are supported
 * 
 * @author icoloma
 */
public class RestAnnotationProcessor extends AbstractAnnotationProcessor {

	private static Log log = Log.getLog(RestAnnotationProcessor.class);
	
	/**
	 * This annotation processor should be executed before any other
	 */
	@Override
	public int getOrder() {
		return 10;
	}
	
	@Override
	public void process(Event event) {
		
		// process class annotations
		ActionMapping mapping = event.getActionMapping();
		Class<? extends Action> actionClass = mapping.getActionClass();
		Path path = actionClass.getAnnotation(Path.class);
		String methodPrefix = "";
		if (path != null) {
			String handle = getHandle(path.value());
			methodPrefix = path.value().substring(handle.length());
			// start and leading slash are fixed by the setHandle() method
			mapping.setHandle(handle);
		}
		
		Method method = event.getJavaMethod();
		String methodMapping = null;
		path = method.getAnnotation(Path.class);
		if (path != null) {
			methodMapping = path.value();
		}
		if (method.getAnnotation(GET.class) != null) {
			event.addMethod(HttpMethod.GET);
		}
		if (method.getAnnotation(HEAD.class) != null) {
			event.addMethod(HttpMethod.HEAD);
		}
		if (method.getAnnotation(POST.class) != null) {
			event.addMethod(HttpMethod.POST);
		}
		if (method.getAnnotation(PUT.class) != null) {
			event.addMethod(HttpMethod.PUT);
		}
		if (method.getAnnotation(DELETE.class) != null) {
			event.addMethod(HttpMethod.DELETE);
		}
		
		// rewrite the event mapping only if it changes
		if (methodPrefix.length() > 0 || methodMapping != null) {
			if (methodMapping == null) {
				methodMapping = event.getUriParser().toString();
			}
			event.setUriParser(UriParser.newInstance(concatMethodMapping(methodPrefix, methodMapping)));
		}
		
		// process the event parameters
		processEventParameters(event);
		
		if (log.isDebugEnabled()) {
			log.debug(mapping.getShortName(), ".", event.getName(), " is mapped at ", mapping.getHandle(), event.getUriParser());
		}
		
	}
	
	protected void processEventParameters(Event event) {
		List<UriToken> tokens = event.getUriParser().getTokens();
		int tokenIndex = 0;
		Set<String> paramNames = Sets.newHashSet();
		int paramIndex = 0;
		for (EventParameter p : event.getEventParameters()) {
			if (p instanceof RequestParametersParameter) {
				return;
			}
			DefaultEventParameter parameter = (DefaultEventParameter) p;
			PathParam pathParam = parameter.getAnnotation(PathParam.class);
			if (pathParam != null) {
				parameter.setName(pathParam.value());
			} 
			QueryParam queryParam = parameter.getAnnotation(QueryParam.class);
			if (queryParam != null) {
				parameter.setName(queryParam.value());
			}
			if (parameter.getName() == null) {
				String name = tokenIndex >= tokens.size()? null : tokens.get(tokenIndex++).getName();
				if (name == null) {
					throw new IllegalArgumentException("Error processing '" + event.getPrintName() + "': No name specified for parameter " + paramIndex + " with type " + parameter.getParameterType().getSimpleName() 
							+ ". Use either @QueryParam or @PathParam");
				}
				parameter.setName(name);
			}
			if (paramNames.contains(parameter.getName())) {
				throw new IllegalArgumentException("Parameter " + parameter.getName() + " specified more than once. Are you using a @QueryParam and @Path position-specified parameters with the same name?");
			}
			paramNames.add(parameter.getName());
			paramIndex++;
		}

	}

	/**
	 * Concatenate a common method prefix (as specified in the action mapping) with this concrete method mapping
	 */
	public String concatMethodMapping(String methodPrefix, String methodMapping) {
		// if there is a double slash, make it single
		if (methodPrefix.endsWith("/") && methodMapping.startsWith("/")) {
			methodMapping = methodMapping.substring(1);
		}
		if (!methodPrefix.endsWith("/") && !methodMapping.startsWith("/")) {
			methodMapping = "/" + methodMapping;
		}
		
		return methodPrefix + methodMapping;
	}

	/**
	 * Extract the handle from the provided action path
	 * @param actionPath the action path specified by the {@link Path} annotation
	 * @return The action handle extracted from the provided path (the substring until the first '/' character) 
	 */
	public String getHandle(String actionPath) {
		String handle = null;
		if (actionPath.length() > 0 && !"/".equals(actionPath)) {
			int pos = actionPath.length() <= 1? -1 : actionPath.indexOf('/', 1);
			handle = pos == -1? actionPath : actionPath.substring(0, pos);
		}
		if (StringUtils.isEmpty(handle) || handle.indexOf('{') != -1) {
			throw new ConfigException("Incorrect mapping '" + actionPath + "'. Action mappings must start with a valid handle without parameters");
		}
		return handle;
	}
	
}
