package org.loom.binding;

import java.util.Collection;

import javax.inject.Inject;
import javax.inject.Singleton;

import org.loom.annotation.CollectionMetadata;
import org.loom.annotation.MapMetadata;
import org.loom.binding.factories.ImplementationFactoriesRepository;
import org.loom.converter.Converter;
import org.loom.converter.ConverterFactory;
import org.loom.converter.LocaleUnawareConverter;
import org.loom.converter.NullConverter;
import org.loom.exception.ConfigException;
import org.loom.util.ClassUtils;

@Singleton
public class ItemMetadataFactory {

	/** the list of configured Implementation Factories */
	@Inject
	private ImplementationFactoriesRepository implementationFactoriesRepository;
	
	@Inject
	private ConverterFactory converterFactory;

	/**
	 * Create item metadata for multiple properties (List, Set, array and Map)
	 * @param container
	 */
	public ItemMetadata[] createItemMetadata(AnnotationContainer container) {
		Class<?> propertyClass = container.getPropertyClass();
		
		if (container.isCollection()) {
			
			Class<?> itemClass = null;
			Class<? extends Converter> converterClass = null;
			
			if (propertyClass.isArray()) { // guess array item class
				itemClass = propertyClass.getComponentType();
			} else if (Collection.class.isAssignableFrom(propertyClass)) { // guess collection generic item type
				itemClass = container.guessCollectionGenericType();
			}
			
			CollectionMetadata cm = container.getAnnotation(CollectionMetadata.class);
			if (cm != null) {
				converterClass = cm.itemConverter();
			} 
			return new ItemMetadata[] { createItemMetadata(converterClass, itemClass) };
			
		} else if (container.isMap()) {
			
			MapMetadata annotation = container.getAnnotation(MapMetadata.class);
			ItemMetadata[] metadata = new ItemMetadata[2];
			
			Class<?> keyClass = container.guessMapGenericKeyType();
			Class<? extends LocaleUnawareConverter> keyConverterClass = annotation == null? null : annotation.keyConverter();
			Class<?> valueClass = container.guessMapGenericValueType();
			Class<? extends Converter> valueConverterClass = annotation == null? null : annotation.valueConverter();
			
			metadata[0] = createItemMetadata(keyConverterClass, keyClass);
			metadata[1] = createMapItemMetadata(valueConverterClass, valueClass);
			return metadata;
			
		}
		return null;
	}

	
	/**
	 * Same as createItemMetadata, but defaults to String (class and converter) if itemClass is null
	 */
	private ItemMetadata createMapItemMetadata(Class<? extends Converter> itemConverterClass, Class<?> itemClass) {
		ItemMetadata itemMetadata = createItemMetadata(itemConverterClass, itemClass);
		if (itemMetadata.getConverter() == null && itemMetadata.getItemClass() == null) { // maps must specify types using generics
			throw new ConfigException("Missing map key/value type. Maybe the generic declaration is missing?");
		}
		return itemMetadata;
	}

	/**
	 * Creates the metadata that may be used to create collection/map nodes.
	 * Any of the arguments may be null, in which case a reasonable default will be applied, if possible.
	 */
	private ItemMetadata createItemMetadata(Class<? extends Converter> converterClass, Class<?> itemClass) {
		Converter converter = null;
		if (converterClass != null && !NullConverter.class.equals(converterClass)) {
			converter = ClassUtils.newInstance(converterClass);
 		} else if (itemClass != null) {
			converter = converterFactory.getConverterForClass(itemClass);
		}
			
		if (converter != null && itemClass == null) {
			itemClass = converter.getConvertedClass();
		} 
		
		ItemMetadata metadata = new ItemMetadata();
		metadata.setItemClass(itemClass);
		metadata.setConverter(converter);
		if (itemClass != null) {
			metadata.setImplementationFactory(implementationFactoriesRepository.getImplementationFactory(itemClass));
		}
		return metadata;
	}


	public void setImplementationFactoriesRepository(
			ImplementationFactoriesRepository implementationFactoriesRepository) {
		this.implementationFactoriesRepository = implementationFactoriesRepository;
	}


	public void setConverterFactory(ConverterFactory converterFactory) {
		this.converterFactory = converterFactory;
	}

}
