/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.binding.factories;

import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.inject.Inject;

import org.apache.commons.lang.StringUtils;
import org.loom.binding.ArrayNode;
import org.loom.binding.ListNode;
import org.loom.binding.MapNode;
import org.loom.binding.Node;
import org.loom.binding.NodeFactory;
import org.loom.binding.PropertyWrapper;
import org.loom.binding.PropertyWrapperFactory;
import org.loom.binding.SetNode;
import org.loom.binding.SimpleNode;
import org.loom.exception.ConfigException;


/**
 * Default NodeFactory implementation
 * @author Ignacio Coloma
 */
public class DefaultNodeFactory implements NodeFactory {

	/** propertyWrapper factory */
	@Inject
	private PropertyWrapperFactory propertyWrapperFactory;
	
	public Node create(Class<?> clazz, String propertyPath) {
		return visit(clazz, StringUtils.split(propertyPath, '.'), 0);
	}
	
	/**
	 * Instantiates a chain of Nodes for a propertyPath
	 * 
	 * @param clazz The class of this node.
	 * @param propertyPath The normalized path to be traversed, e.g. [ "customer", "street", "name" ] for customer.street.name
	 * @param index the current node index
	 * @return The chain of Nodes for this index and ahead
	 */
	public Node visit(Class<?> clazz, String[] propertyPath, int index) {
		String propertyName = propertyPath[index];
		if (clazz == null) {
			throw new ConfigException("Cannot get value of '" + propertyName + "' in property path '" + StringUtils.join(propertyPath, '.') + 
					"' because its container class has not been specified. Please consider using @CollectionMetadata.itemClass or @MapMetadata.valueClass");
		}
		PropertyWrapper propertyWrapper = propertyWrapperFactory.createInstance(clazz, propertyName);
		Node node = createNode(propertyWrapper);
		if (index < propertyPath.length - 1) {
			Node child = visit(node.getNextNodeClass(), propertyPath, index + 1);
			node.setChild(child);
		} 
		return node;
	}

	public Node createNode(PropertyWrapper propertyWrapper) {
		assert propertyWrapper != null : "PropertyWrapper cannot be null";
		Class<?> propertyClass = propertyWrapper.getPropertyClass();
		if (List.class.isAssignableFrom(propertyClass)) {
			return createListNode(propertyWrapper);
		} else if (Map.class.isAssignableFrom(propertyClass)) {
			return createMapNode(propertyWrapper);
		} else if (Set.class.isAssignableFrom(propertyClass)) {
			return createSetNode(propertyWrapper);
		} else if (propertyClass.isArray() && !Byte.TYPE.equals(propertyClass.getComponentType())) {
			return createArrayNode(propertyWrapper);
		} else { 
			return createSimpleNode(propertyWrapper);
		}
	}

	private Node createSimpleNode(PropertyWrapper propertyWrapper) {
		SimpleNode node = new SimpleNode();
		node.setPropertyWrapper(propertyWrapper);
		return node;
	}

	private Node createArrayNode(PropertyWrapper propertyWrapper) {
		ArrayNode node = new ArrayNode();
		node.setPropertyWrapper(propertyWrapper);
		return node;
	}

	private Node createListNode(PropertyWrapper propertyWrapper) {
		ListNode node = new ListNode();
		node.setPropertyWrapper(propertyWrapper);
		return node;
	}
	
	private Node createSetNode(PropertyWrapper propertyWrapper) {
		SetNode node = new SetNode();
		node.setPropertyWrapper(propertyWrapper);
		return node;
	}

	private Node createMapNode(PropertyWrapper propertyWrapper) {
		MapNode node = new MapNode();
		node.setPropertyWrapper(propertyWrapper);
		return node;
	}

	public void setPropertyWrapperFactory(
			PropertyWrapperFactory propertyWrapperFactory) {
		this.propertyWrapperFactory = propertyWrapperFactory;
	}

}
