/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.interceptor;

import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.loom.i18n.Messages;
import org.loom.mapping.ParsedAction;
import org.loom.resolution.Resolution;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletRequestImpl;
import org.loom.servlet.params.FileParameter;
import org.loom.util.HtmlSanitizer;
import org.loom.util.StringUtils;


/**
 * Validate one file parameter
 * This class is thread-safe.
 * 
 * @author Rafael Serrano
 */
public class FileValidationInterceptor implements ValidateInterceptor {

	/** allowed formats */
	protected Set<String> allowedFormats = new TreeSet<String>();

    /** maximum file size */
    protected Integer maxFileSize;
    
    /** the parameter to validate */
    protected String parameterName;
    
    public final Resolution beforeValidate(ParsedAction action) {
		LoomServletRequest request = LoomServletRequestImpl.getThreadLocal();
		validateParameter(request, parameterName, request.getFileParameter(parameterName));
		return null;
    }

    /**
     * Validate one file parameter. Return true if it passed, false if not.
     */
	protected void validateParameter(LoomServletRequest request, String parameterName, FileParameter fileParam) {
		if (fileParam == null) {
			return;
		}
		
		// check if the size is too big
		Messages messages = request.getParsedAction().getMessages();
		if (maxFileSize != null) {
			if (fileParam.getFileSize() > maxFileSize) {
				messages.error(parameterName, "loom.validation.fileSizeTooLarge")
					.addArg("maxFileSize", maxFileSize)
					.addArg("filename", HtmlSanitizer.sanitize(fileParam.getFilename()));
				return;
			}
		}
		
		// check file extension
		if (!allowedFormats.isEmpty()) {
			String extension = StringUtils.substringAfterLast(fileParam.getFilename(), ".");
			if (StringUtils.isEmpty(extension) || !allowedFormats.contains(extension)) {
				messages.error(parameterName, "loom.validation.fileExtensionFailed")
					.addArg("filename", HtmlSanitizer.sanitize(fileParam.getFilename()))
					.addArg("validFormats", StringUtils.join(allowedFormats, ", "));
				return;
			}
		}
		
		validateParameterExtensionPoint(request, parameterName, fileParam);
		
	}
	
	/**
	 * Extension point to perform additional validations
	 */
	protected void validateParameterExtensionPoint(LoomServletRequest request, String parameterName, FileParameter fileParam) {
		// empty
	}

    @Override
    public String toString() {
    	return new ToStringBuilder(this)
    		.append("parameterName", parameterName)
    		.append("allowedFormats", allowedFormats)
    		.toString();
    }

	public void setAllowedFormats(String[] allowedFormats) {
		for (String format : allowedFormats) {
			this.allowedFormats.add(format.toLowerCase());
		}
	}

	public Integer getMaxFileSize() {
		return maxFileSize;
	}

	public void setMaxFileSize(Integer maxFileSize) {
		this.maxFileSize = maxFileSize;
	}

	public void setAllowedFormats(Set<String> allowedFormats) {
		this.allowedFormats = allowedFormats;
	}

	public void setParameterName(String parameterName) {
		this.parameterName = parameterName;
	}

	public Set<String> getAllowedFormats() {
		return allowedFormats;
	}

	public String getParameterName() {
		return parameterName;
	}
	
}
