package org.loom.mapping;

import java.lang.annotation.Annotation;
import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.loom.binding.AnnotationContainer;
import org.loom.binding.ItemMetadata;
import org.loom.binding.PropertyBinder;
import org.loom.binding.PropertyBinderFactory;
import org.loom.converter.Converter;
import org.loom.i18n.Messages;
import org.loom.i18n.MessagesRepository;
import org.loom.log.Log;
import org.loom.servlet.params.ServletRequestParameters;
import org.loom.util.ClassUtils;
import org.loom.util.PropertyUtils;
import org.loom.validator.PropertyValidator;
import org.loom.validator.ValidationRequest;
import org.springframework.core.GenericCollectionTypeResolver;
import org.springframework.core.MethodParameter;

import com.google.common.collect.LinkedHashMultimap;
import com.google.common.collect.SetMultimap;

public class DefaultEventParameter implements EventParameter {

	/** the argument name */
	private String name;

	/** for simple parameters, the {@link Converter} instance to be used */
	private Converter converter;

	/**
	 * the list of item metadata specified by SetMetadata, CollectionMetadata or
	 * MapMetadata. It is handled in the same order as generics parameters.
	 * Lists have a single value that corresponds to the list item type, and
	 * Maps have two values that are the key type and the value type,
	 * respectively.
	 */
	private ItemMetadata[] itemMetadata;

	/** the parameter class */
	protected Class<?> parameterType;

	/** for complex types, the properties that can be populated */
	protected Map<String, PropertyBinder> properties;

	protected PropertyBinderFactory propertyBinderFactory;

	/** the list of validators to execute for this parameter */
	protected SetMultimap<String, PropertyValidator> parameterValidators = LinkedHashMultimap.create();

	/** the list of annotations set on this parameter */
	protected Annotation[] annotations;

	/** the event */
	protected Event event;
	
	private static Log log = Log.getLog(DefaultEventParameter.class);

	/**
	 * If the parameter has the provided annotation, return it. Else, return
	 * null.
	 */
	@SuppressWarnings("unchecked")
	public <T extends Annotation> T getAnnotation(Class<T> annotationClass) {
		for (Annotation annotation : annotations) {
			if (annotationClass.isAssignableFrom(annotation.getClass())) {
				return (T) annotation;
			}
		}
		return null;
	}

	/** the zero-based parameter index */
	protected int index;

	public Event getEvent() {
		return event;
	}

	public void setEvent(Event event) {
		this.event = event;
	}

	public int getIndex() {
		return index;
	}

	public void setIndex(int index) {
		this.index = index;
	}

	public void setPropertyBinderFactory(
			PropertyBinderFactory propertyBinderFactory) {
		this.propertyBinderFactory = propertyBinderFactory;
	}

	@Override
	public void addValidator(PropertyValidator validator) {
		parameterValidators.put(validator.getPropertyPath(), validator);
	}

	@Override
	public Set<PropertyValidator> getValidators(String propertyPath) {
		return parameterValidators.get(propertyPath);
	}

	public void setAnnotations(Annotation[] annotations) {
		this.annotations = annotations;
	}

	public Annotation[] getAnnotations() {
		return annotations;
	}

	public Class<?> getParameterType() {
		return parameterType;
	}

	/**
	 * this method exists only to comply with the implemented interface
	 * {@link AnnotationContainer}
	 */
	public Class getPropertyClass() {
		return getParameterType();
	}

	public void setParameterType(Class<?> parameterType) {
		this.parameterType = parameterType;
	}

	public Class<?> guessCollectionGenericType() {
		return GenericCollectionTypeResolver
				.getCollectionParameterType(new MethodParameter(event
						.getJavaMethod(), index));
	}

	public Class<?> guessMapGenericKeyType() {
		return GenericCollectionTypeResolver
				.getMapKeyParameterType(new MethodParameter(event
						.getJavaMethod(), index));
	}

	public Class<?> guessMapGenericValueType() {
		return GenericCollectionTypeResolver
				.getMapValueParameterType(new MethodParameter(event
						.getJavaMethod(), index));
	}

	public boolean isCollection() {
		return Collection.class.isAssignableFrom(parameterType)
				|| parameterType.isArray();
	}

	public boolean isMap() {
		return Map.class.isAssignableFrom(parameterType);
	}

	@Override
	public String toString() {
		return new ToStringBuilder(this).append("name", name)
				.append("parameterType", parameterType.getName())
				.append("converter", converter)
				.append("annotations", annotations).toString();
	}

	@Override
	public Object populate(Object container, ServletRequestParameters parameters, MessagesRepository repository, Messages messages) {

		// if it's a simple parameter
		if (converter != null) {
			String stringValue = parameters.getStringParameter(name);
			parameters.setAssigned(name);
			return converter.getAsObject(name, stringValue, messages,
					repository);
		}

		// complex objects
		for (Map.Entry<String, String> entry : parameters
				.getStringParametersEntrySet()) {
			String paramName = entry.getKey();
			if (paramName.startsWith(name)) {
				log.trace("Setting ", paramName);
				if (container == null) {
					container = ClassUtils.newInstance(parameterType);
				}
				String paramValue = entry.getValue();
				PropertyBinder propertyBinder = getPropertyBinder(PropertyUtils
						.normalize(paramName));
				Object valueAsObject = propertyBinder.getConverter()
						.getAsObject(paramName, paramValue, messages,
								repository);
				propertyBinder.setValue(container,
						StringUtils.removeStart(paramName, name + "."),
						valueAsObject);
				parameters.setAssigned(paramName);
			}
		}
		return container;
	}

	public void validate(Object container, Messages messages) {
		for (PropertyValidator validator : parameterValidators.values()) {
			String normalizedPropertyPath = validator.getPropertyPath();
			if (!messages.hasAnyError(normalizedPropertyPath)) { // do not
																	// overwrite
																	// previous
																	// (conversion)
																	// errors
				log.trace("Validating ", normalizedPropertyPath);
				ValidationRequest validationRequest = new ValidationRequest(
						validator, messages).push(name);
				if (normalizedPropertyPath.length() == name.length()) { // simple
																		// parameter
					validationRequest.validate(container);
				} else { // composite parameter
					PropertyBinder binder = getPropertyBinder(normalizedPropertyPath);
					binder.applyValidation(container, validationRequest);
				}
			}
		}
	}

	public PropertyBinder getPropertyBinder( String normalizedPropertyPathWithPrefix) {
		String normalizedPropertyPath = normalizedPropertyPathWithPrefix
				.substring(name.length() + 1);
		PropertyBinder result = properties.get(normalizedPropertyPath);
		if (result != null) {
			return result;
		}
		result = propertyBinderFactory.create(parameterType,
				normalizedPropertyPath);
		properties.put(normalizedPropertyPath, result);
		return result;
	}

	public boolean matches(String propertyPath) {
		return propertyPath.startsWith(name)
				&& (propertyPath.length() == name.length() || properties != null
						&& propertyPath.charAt(name.length()) == '.');
	}

	public Object getPropertyAsObject(Object container, String propertyPath) {
		if (propertyPath.length() == name.length()) {
			return container;
		}
		PropertyBinder binder = getPropertyBinder(PropertyUtils
				.normalize(propertyPath));
		return binder.getValue(container,
				propertyPath.substring(name.length() + 1));
	}

	public String getPropertyAsString(Object container, String propertyPath, MessagesRepository repository) {
		if (propertyPath.length() == name.length()) {
			return converter.getAsText(container, repository);
		}
		PropertyBinder binder = getPropertyBinder(PropertyUtils
				.normalize(propertyPath));
		return binder.getValueAsText(container,
				propertyPath.substring(name.length() + 1), repository);

	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Converter getConverter() {
		return converter;
	}

	public void setConverter(Converter converter) {
		this.converter = converter;
		if (converter == null) {
			properties = new ConcurrentHashMap<String, PropertyBinder>();
		}
	}

	public String getDisplayName() {
		return "event parameter " + name;
	}

	/**
	 * Assigns value to some property nested inside this eventParameter
	 */
	public void setPropertyAsObject(Map<String, Object> parameterValues, String propertyPath, Object value) {
		if (name.length() == propertyPath.length()) {
			parameterValues.put(propertyPath, value);
		} else {
			Object rootParameterValue = parameterValues.get(name);
			if (rootParameterValue == null && value != null) {
				rootParameterValue = ClassUtils.newInstance(parameterType);
				parameterValues.put(name, rootParameterValue);
			}
			PropertyBinder propertyBinder = getPropertyBinder(PropertyUtils
					.normalize(propertyPath));
			propertyBinder.setValue(rootParameterValue,
					propertyPath.substring(name.length() + 1), value);
		}
	}
	public void setItemMetadata(ItemMetadata[] itemMetadata) {
		this.itemMetadata = itemMetadata;
	}

	public ItemMetadata getItemMetadata(int index) {
		return itemMetadata == null ? null : itemMetadata[index];
	}

	public int getItemMetadataSize() {
		return itemMetadata == null ? 0 : itemMetadata.length;
	}

}
