/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.mapping;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.loom.binding.PropertyBinder;
import org.loom.binding.PropertyBinderMap;
import org.loom.converter.Converter;
import org.loom.exception.ConfigException;
import org.loom.exception.MalformedPropertyPathException;
import org.loom.exception.ParameterException;
import org.loom.i18n.Messages;
import org.loom.interceptor.Interceptor;
import org.loom.interceptor.InterceptorProxy;
import org.loom.log.Log;
import org.loom.resolution.Resolution;
import org.loom.servlet.HttpMethod;
import org.loom.servlet.LoomServletRequest;
import org.loom.tags.Button;
import org.loom.tags.Form;
import org.loom.tags.Link;
import org.loom.validator.ButtonDecorator;
import org.loom.validator.FormDecorator;
import org.loom.validator.LinkDecorator;
import org.loom.validator.PathMatcher;
import org.loom.validator.PropertyValidator;
import org.springframework.core.annotation.AnnotationUtils;

import com.google.common.collect.LinkedHashMultimap;
import com.google.common.collect.ListMultimap;
import com.google.common.collect.Lists;
import com.google.common.collect.SetMultimap;
import com.google.common.collect.Sets;

/**
 * This class holds the internal data associated to an event.
 * Events are public methods in Action classes that return {@link Resolution}
 * and receive no arguments. 
 * @author Ignacio Coloma
 */
public class Event implements LinkDecorator, FormDecorator, ButtonDecorator {

	/** the actionMapping that contains this event */
	private ActionMapping actionMapping;
	
	/** the associated java method */
	private Method javaMethod;
	
	/** the list of interceptors to be run */
	private InterceptorProxy interceptorProxy = new InterceptorProxy();
	
	/** the list of Validators bound to class attribute to apply for this event. The propertypath is the key */
	private SetMultimap<String, PropertyValidator> attributeValidators = LinkedHashMultimap.create();
	
	/** the list of {@link HttpMethod} allowed to invoke this event, default GET and POST */
	private Set<HttpMethod> httpMethods = DEFAULT_ALLOWED_METHODS;
	
	/** the URI parser */
	private UriParser uriParser;
	
	/** the list of arguments acepted by the java method */
	private List<EventParameter> eventParameters = Lists.newArrayList();
	
	private static Log log = Log.getLog(Event.class);
	
	/** the list of default http methods to be applied unless otherwise specified */
	private static Set<HttpMethod> DEFAULT_ALLOWED_METHODS;
	
	static {
		DEFAULT_ALLOWED_METHODS = new HashSet<HttpMethod>();
		DEFAULT_ALLOWED_METHODS.add(HttpMethod.GET);
		DEFAULT_ALLOWED_METHODS.add(HttpMethod.POST);
	}
	
	public Event(ActionMapping mapping, Method method) {
		this.actionMapping = mapping;
		this.javaMethod = method;
	}
	
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("Event ").append(getPrintName()).append(" { ").append(uriParser);
		if (!interceptorProxy.isEmpty()) {
			builder.append(", interceptors = ").append(interceptorProxy.toString());
		}
		if (!attributeValidators.isEmpty()) {
			builder.append(", propertyValidators = ").append(attributeValidators.values());
		}
		builder.append(" }");
		return builder.toString();
			
	}

	/**
	 * Return the property class of a normalized property path.
	 * Handles transparently if the property is nested inside a method arguyment or is an action attribute.
	 * @param normalizedPropertyPath the entire property path
	 */
	public Class<?> getPropertyClass(String normalizedPropertyPath) {
		if (!eventParameters.isEmpty()) {
			for (EventParameter eventParameter : eventParameters) {
				if (eventParameter.matches(normalizedPropertyPath)) {
					if (normalizedPropertyPath.length() == eventParameter.getName().length()) { // simple parameter
						return eventParameter.getParameterType();
					} else {
						return eventParameter.getPropertyBinder(normalizedPropertyPath).getPropertyClass();
					}
				}
			}
		}
		return actionMapping.getPropertyBinder(normalizedPropertyPath).getPropertyClass();
	}
	
	/**
	 * @return the converter for the provided normalizedPropertyPath, null if none
	 */
	public Converter getConverter(String normalizedPropertyPath) {
		if (!eventParameters.isEmpty()) {
			for (EventParameter eventParameter : eventParameters) {
				if (eventParameter.matches(normalizedPropertyPath)) {
					if (normalizedPropertyPath.length() == eventParameter.getName().length()) { // simple parameter
						return eventParameter.getConverter();
					} else {
						return eventParameter.getPropertyBinder(normalizedPropertyPath).getConverter();
					}
				}
			}
		}
		return actionMapping.getConverter(normalizedPropertyPath);
	}
	
	/**
	 * If this event is a match, return the list of parameter values extracted from the uri. 
	 * If the URI is not a match or the HTTP method is not included in the list of allowed methods, 
	 * return null.
	 * @param uri the URI to be parsed, without the action handle.
	 * @param request the request object
	 * @return the Map of parameters extracted from the uri, null if it's  not a match.
	 */
	public Map<String, String> parse(String uri, LoomServletRequest request) {
		if (!httpMethods.contains(request.getHttpMethod())) {
			return null;
		}
		return uriParser.parse(uri);
	}

	/**
	 * @return the event name. This is the same as the method name.
	 */
	public String getName() {
		return javaMethod.getName();
	}
	
	/**
	 * @return the print (full) event name for error messages. This is the action short name followed by the event name. 
	 */
	public String getPrintName() {
		return actionMapping.getShortName() + "." + getName();
	}

	/**
	 * Adds a new parameter or attribute Validator.
	 * Allows validation overriding. If this property already has a validation of the same type,
 	 * override it with the new one (for example, override the validation in 
 	 * fooAction.barField using annotations in fooAction).
	 * @param validator
	 */
	public void addValidator(PropertyValidator validator) {
		log.trace("Registering ", validator, " for event ", getName());
		for (EventParameter eventParameter : eventParameters) {
			if (eventParameter.matches(validator.getPropertyPath())) {
				eventParameter.addValidator(validator);
				return;
			}
		}
		
		Class<? extends PropertyValidator> validatorClass = validator.getClass();
		String propertyPath = validator.getPropertyPath();
		for (Iterator<PropertyValidator> i = attributeValidators.get(propertyPath).iterator(); i.hasNext();) {
			PropertyValidator existing = i.next();
			if (existing.getClass().equals(validatorClass)) {
				if (log.isDebugEnabled()) {
					log.debug("Overriding ", existing, " for attribute ", propertyPath, " at event ", javaMethod.getName());
				}
				i.remove();
				break;
			}
		}
		attributeValidators.put(propertyPath, validator);
	}
	
	/**
	 * Applies all the configured validations to this object
	 * @param action the action and list of error messages
	 */
	public void validate(ParsedAction action) throws MalformedPropertyPathException {
		Messages messages = action.getMessages();
		PropertyBinderMap propertyBinderMap = actionMapping.getPropertyBinderMap();
		
		// validate event parameters
		for (EventParameter eventParameter : eventParameters) {
			eventParameter.validate(action.getEventParameterValue(eventParameter.getName()), messages);
		}
		
		// validate action attributes
		for (PropertyValidator validator : attributeValidators.values()) {
			String normalizedPropertyPath = validator.getPropertyPath();
			if (!messages.hasAnyError(normalizedPropertyPath)) { // do not overwrite previous (conversion) errors
				log.trace("Validating ", normalizedPropertyPath);
				PropertyBinder binder = propertyBinderMap.getRequiredPropertyBinder(normalizedPropertyPath);
				binder.applyValidation(action.getAction(), validator, messages);
			}
		}
	}

	/**
	 * @throws ConfigException if this Event is mapped to a URL that overlaps with another event for the same HTTP method
	 */
	public void assertNoOverlapping(Event other) {
		Set<HttpMethod> commonMethods = Sets.intersection(this.httpMethods, other.getHttpMethods());
		if (!commonMethods.isEmpty() && uriParser.overlaps(other.getUriParser())) {
				throw new ConfigException("Event " + getPrintName() + "(" + uriParser + ") overlaps with " + other.getPrintName() + 
						" (" + other.getUriParser() + ") for method " + commonMethods + ". Either change the http method or the mapping URI");
		}
	}

	/**
	 * Prepare the string representatuion of this method with the provided params.
	 * @param params the list of parameters to include in the URL. 
	 * @return the String representation of this event object
	 */
	public StringBuilder toUrl(ListMultimap<String, String> params) {
		try {
			StringBuilder result = new StringBuilder(32);
			result.append(actionMapping.getHandle());
			uriParser.format(result, params);
			return result;
		} catch (ParameterException e) {
			throw new ParameterException(getPrintName() + ": " + e.getMessage(), e);
		}

	}
	
	/**
	 * @return a list of the parameter names that are used to build the uri
	 */
	public Set<String> getUriParameterNames() {
		return uriParser.getParameterNames();
	}
	
	public void addInterceptor(Interceptor interceptor) {
		log.trace("Registering ", interceptor, " for event ", getName());
		interceptorProxy.addListener(interceptor);
	}
	
	public Method getJavaMethod() {
		return javaMethod;
	}

	public InterceptorProxy getInterceptors() {
		return interceptorProxy;
	}

	public void setInterceptorList(InterceptorProxy interceptorList) {
		this.interceptorProxy = interceptorList;
	}
	
	/**
	 * Used only to inspect the list of validators while in development (actionDetails.jsp).
	 * This method should not be used for any other purpose, as it exposes some class internals.
	 * @deprecated this method will be removed with no replacement
	 */
	public Collection<PropertyValidator> getValidators() {
		return Collections.unmodifiableCollection(attributeValidators.values());
	}
	
	/**
	 * @param propertyPath the property path to retrieve validators 
	 * @return Validators registered with the provided propertyName
	 */
	public Set<PropertyValidator> getValidatorsFor(String propertyPath) {
		for (EventParameter parameter : eventParameters) {
			if (parameter.matches(propertyPath)) {
				return parameter.getValidators(propertyPath);
			}
		}
		return attributeValidators.get(propertyPath);
	}

	public ActionMapping getActionMapping() {
		return actionMapping;
	}

	/**
	 * @return true if this is the default event
	 */
	public boolean isDefault() {
		return actionMapping.getDefaultEvent() == this;
	}

	/**
	 * Modify link attributes based on the interceptors bound to this event.
	 */
	public void decorate(Link link) {
		interceptorProxy.decorate(link);
		if (link.getMethod() == null && !this.httpMethods.contains(HttpMethod.GET)) {
			link.setMethod(this.httpMethods.iterator().next());
		}
	}
	
	/**
	 * Modify form attributes based on the interceptors bound to this event.
	 */
	public void decorate(Form form) {
		interceptorProxy.decorate(form);
	}

	/**
	 * Modify button attributes based on the interceptors bound to this event.
	 */
	public void decorate(Button button) {
		interceptorProxy.decorate(button);
	}

	/**
	 * @return true if this method supports (can be invoked with) the provided HTTP method
	 */
	public boolean supports(HttpMethod httpMethod) {
		return httpMethods.contains(httpMethod);
	}
	
	/**
	 * Add an allowed http method to this event.
	 * By default, GET and POST are allowed unless otherwise specified.
	 * @param method the method that should be allowed
	 */
	public void addMethod(HttpMethod method) {
		if (httpMethods == DEFAULT_ALLOWED_METHODS) {
			httpMethods = new LinkedHashSet<HttpMethod>();
		}
		httpMethods.add(method);
	}

	public UriParser getUriParser() {
		return uriParser;
	}

	public void setUriParser(UriParser uriParser) {
		this.uriParser = uriParser;
	}

	public Set<HttpMethod> getHttpMethods() {
		return httpMethods;
	}

	public List<EventParameter> getEventParameters() {
		return eventParameters;
	}

	public void addEventParameter(EventParameter parameter) {
		eventParameters.add(parameter);
	}

	/**
	 * Return true if this event matches the "on", "except" clauses of the provided annotation instance
	 */
	public <T extends Annotation> boolean matches(T annotation) {
		return new PathMatcher(
				(String[])AnnotationUtils.getValue(annotation, "on"), 
				(String[])AnnotationUtils.getValue(annotation, "except")
				).matches(this.getName());
	}

}
