/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.mapping;

import java.util.HashMap;
import java.util.Map;

import org.loom.action.Action;
import org.loom.binding.PropertyBinderMap;
import org.loom.exception.WorkflowInterruption;
import org.loom.i18n.Messages;
import org.loom.i18n.MessagesRepository;
import org.loom.log.Log;
import org.loom.resolution.Resolution;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.servlet.Phase;
import org.loom.servlet.params.ServletRequestParameters;
import org.loom.url.UrlBuilder;
import org.loom.util.ClassUtils;

/**
 * The current action and event object, as parsed from the URL
 * @author icoloma
 */
public class ParsedAction {

	/** the action instance with all the properties populated */
	private Action action;
	
	/** the event that is being called, or the default event if not specified (guaranteed not to be null) */
	private Event event;
	
	/** the errors occurred during conversion and validation of properties */
	private Messages messages;
	
	/** the {@link ActionMapping} instance corresponding to this Action */
	private ActionMapping actionMapping;
	
	/** the current request object */
	private LoomServletRequest request;
	
	/** the current response object */
	private LoomServletResponse response;
	
	/** the current event parameter values, as they will be passed to the event method */
	private Map<String, Object> eventParameterValues = new HashMap<String, Object>();
	
	private static Log log = Log.getLog(ParsedAction.class);
	
	public ParsedAction(ActionMapping actionMapping, Action action, Event event, Messages messages, LoomServletRequest request, LoomServletResponse response) {
		this.actionMapping = actionMapping;
		this.action = action;
		this.event = event == null? actionMapping.getDefaultEvent() : event;
		this.messages = messages;
		this.request = request;
		this.response = response;
	}

	/**
	 * Injects the request values into the action. Will skip any parameter that has been marked as already processed (by a prior phase)
	 * @param parameters the list of processed values from the LoomServletRequest
	 */
	public void populate(ServletRequestParameters parameters) {
		// initialize event parameter values
		for (EventParameter eventParameter : event.getEventParameters()) {
			String parameterName = eventParameter.getName();
			Object populatedValue = eventParameter.populate(eventParameterValues.get(parameterName), parameters, request.getMessagesRepository(), messages);
			eventParameterValues.put(parameterName, populatedValue);
		}
		
		// initialize action properties
		PropertyBinderMap properties = actionMapping.getPropertyBinderMap();
		properties.populate(action, parameters, request.getMessagesRepository(), messages);
	}
	
	/**
	 * @return the required property value in the current parsed action
	 */
	public String getPropertyAsString(String propertyPath) {
		MessagesRepository repository = request.getMessagesRepository();
		
		for (EventParameter eventParameter: event.getEventParameters()) {
			if (eventParameter.matches(propertyPath)) {
				Object parameterValue = eventParameterValues.get(eventParameter.getName());
				return eventParameter.getPropertyAsString(parameterValue, propertyPath, repository);
			}
		}
			
		return actionMapping.getValueAsString(action, propertyPath, repository);
	}
	
	/**
	 * @return the required property value in the current parsed action
	 */
	public Object getPropertyAsObject(String propertyPath) {
		for (EventParameter eventParameter: event.getEventParameters()) {
			if (eventParameter.matches(propertyPath)) {
				Object parameterValue = eventParameterValues.get(eventParameter.getName());
				return eventParameter.getPropertyAsObject(parameterValue, propertyPath);
			}
		}
		return actionMapping.getValueAsObject(action, propertyPath, request.getMessagesRepository());
	}
	
	public void setPropertyAsObject(String propertyPath, Object value) {
		for (EventParameter eventParameter: event.getEventParameters()) {
			if (eventParameter.matches(propertyPath)) {
				eventParameter.setPropertyAsObject(eventParameterValues, propertyPath, value);
				return;
			}
		}
		actionMapping.setPropertyAsObject(action, propertyPath, value);
	}
	
	public void afterCreate() throws WorkflowInterruption {
		log.trace("Firing afterCreate()");
		Resolution result = event.getInterceptors().afterCreate(this);
		if (result != null) {
			throw new WorkflowInterruption(Phase.CREATE, result);
		}
	}
	
	public void beforePopulate()  throws WorkflowInterruption {
		log.trace("Firing beforePopulate()");
		Resolution result = event.getInterceptors().beforePopulate(this);
		if (result != null) {
			throw new WorkflowInterruption(Phase.POPULATE, result);
		}
	}
	
	public void beforeValidate() throws WorkflowInterruption {
		log.trace("Firing beforeValidate()");
		Resolution result = event.getInterceptors().beforeValidate(this);
		if (result != null) {
			throw new WorkflowInterruption(Phase.VALIDATE, result);
		}
	}
	
	public void beforeExecute() throws WorkflowInterruption {
		log.trace("Firing beforeExecute()");
		Resolution result = event.getInterceptors().beforeExecute(this);
		if (result != null) {
			throw new WorkflowInterruption(Phase.EXECUTE, result);
		}
	}
	
	public void beforeResolution(Resolution resolution) {
		log.trace("Firing beforeResolution()");
		event.getInterceptors().beforeResolution(this, resolution);
	}
	
	public void onError()  throws WorkflowInterruption {
		log.trace("Firing onError() event");
		Resolution result = event.getInterceptors().onError(this);
		if (result != null) {
			throw new WorkflowInterruption(Phase.CHECK_ERRORS, result);
		}
	}
	
	public void doFinally(Exception e) {
		log.trace("Firing doFinally() event");
		event.getInterceptors().doFinally(this, e);
	}
	
	public Action getAction() {
		return action;
	}

	public Event getEvent() {
		return event;
	}

	public void setAction(Action action) {
		this.action = action;
	}

	public void setEvent(Event event) {
		this.event = event;
	}

	public Messages getMessages() {
		return messages;
	}

	public void setMessages(Messages messages) {
		this.messages = messages;
	}

	public ActionMapping getActionMapping() {
		return actionMapping;
	}

	/**
	 * @param name the name of the input tag (the property path)
	 * @return true if the component has any associated error
	 */
	public boolean hasAnyError(String name) {
		return messages == null? false : messages.hasAnyError(name);
	}
	
	/**
	 * @return true if exists any error
	 */
	public boolean hasAnyError() {
		return messages == null? false : ! messages.isEmpty();
	}

	/**
	 * @return true if the target action class is the same or a subclass of this one.
	 * /
	public boolean isAssignableFrom(TargetAction targetAction) {
		Class<?> targetActionClass = targetAction.getActionMapping().getActionClass();
		return action.getClass().isAssignableFrom(targetActionClass);
	}*/

	public LoomServletRequest getRequest() {
		return request;
	}
	
	/**
	 * Invokes an event inside an action
	 */
	public Resolution invokeEvent() {
		Object[] values = new Object[event.getEventParameters().size()];
		int i = 0;
		for (EventParameter parameter : event.getEventParameters()) {
			values[i++] = eventParameterValues.get(parameter.getName());
		}
		return (Resolution) ClassUtils.invoke(action, event.getJavaMethod(), values);
	}

	public LoomServletResponse getResponse() {
		return response;
	}

	/**
	 * A ParsedAction matches a UrlBuilder if it has the same action and event and all the 
	 * parameters included in the UrlBuilder have the same value.
	 * Note that the ParsedAction can have MORE parameters than the ones included in the UrlBuilder,
	 * and still return true.
	 * @return true if this action matches the provided UrlBuilder.
	 */
	public boolean matches(UrlBuilder urlBuilder) {
		if (!matches(urlBuilder.getActionMapping(), urlBuilder.getEvent().getName())) {
			return false;
		}
		for (Map.Entry<String, String> param : urlBuilder.getParameters().entries()) {
			String actualValue = request.getStringParameter(param.getKey());
			if (!param.getValue().equals(actualValue)) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * A ParsedAction matches the provided params if it has the same action and event that are passed as parameters
	 * @return true if action and event match
	 */
	public boolean matches(ActionMapping candidateAction, String candidateEvent) {
		if (!(actionMapping.equals(candidateAction))) {
			return false;
		}
		return event.getName().equals(candidateEvent) || event.isDefault() && candidateEvent == null;
	}

	public Map<String, Object> getEventParameterValues() {
		return eventParameterValues;
	}

	public void setEventParameterValues(Map<String, Object> eventParameterValues) {
		this.eventParameterValues = eventParameterValues;
	}

	public Object getEventParameterValue(String name) {
		return eventParameterValues.get(name);
	}

	public boolean matches(String candidateAction, String candidateEvent) {
		if (!(actionMapping.getShortName().equals(candidateAction))) {
			return false;
		}
		return event.getName().equals(candidateEvent) || event.isDefault() && candidateEvent == null;
	}

}
