package org.loom.mapping;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.loom.annotation.RequestParameters;
import org.loom.binding.PropertyBinder;
import org.loom.converter.Converter;
import org.loom.exception.ConfigException;
import org.loom.i18n.Messages;
import org.loom.i18n.MessagesRepository;
import org.loom.log.Log;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.servlet.params.ServletRequestParameters;
import org.loom.util.ClassUtils;
import org.loom.util.PropertyUtils;

/**
 * Event Parameter that contains the full request contents. This parameter is generated 
 * when an Event specifies a {@link RequestParameters}
 * @author icoloma
 *
 */
public class RequestParametersParameter extends DefaultEventParameter {

	private static Log log = Log.getLog(RequestParametersParameter.class);

	/** the fictional name of this parameter (should rarely be used) */
	private static String PARAM_NAME = "__req__";
	
	public RequestParametersParameter() {
		properties = new ConcurrentHashMap<String, PropertyBinder>();
	}
	
	@Override
	public Object populate(Object container, ServletRequestParameters parameters, MessagesRepository repository, Messages messages) {
		
		// complex objects
		for (Map.Entry<String, String> entry : parameters.getStringParametersEntrySet()) {
			String paramName = entry.getKey();
			if (!paramName.startsWith(RequestParameterNames.LOOM_PREFIX)) {
				log.trace("Setting ", paramName);
				if (container == null) {
					container = ClassUtils.newInstance(getParameterType());
				}
				String paramValue = entry.getValue();
				PropertyBinder propertyBinder = getPropertyBinder(PropertyUtils.normalize(paramName));
				Object valueAsObject = propertyBinder.getConverter().getAsObject(paramName, paramValue, messages, repository);
				propertyBinder.setValue(container, paramName, valueAsObject);
				parameters.setAssigned(paramName);
			}
		}
		return container;
	}
	
	@Override
	public void setPropertyAsObject(Map<String, Object> parameterValues, String propertyPath, Object value) {
		Object rootParameterValue;
		if (parameterValues.isEmpty()) {
			rootParameterValue = ClassUtils.newInstance(parameterType);
			parameterValues.put(PARAM_NAME, rootParameterValue);
		} else {
			rootParameterValue = parameterValues.values().iterator().next();
		}
		PropertyBinder propertyBinder = getPropertyBinder(PropertyUtils.normalize(propertyPath));
		propertyBinder.setValue(rootParameterValue, propertyPath, value);
	}

	/**
	 * @param normalizedPropertyPath the property path 
	 */
	@Override
	public PropertyBinder getPropertyBinder(String normalizedPropertyPath) {
		PropertyBinder result = properties.get(normalizedPropertyPath);
		if (result != null) {
			return result;
		}
		result = propertyBinderFactory.create(parameterType, normalizedPropertyPath);
		properties.put(normalizedPropertyPath, result);
		return result;
	}

	@Override
	public void setConverter(Converter converter) {
		if (converter != null) {
			throw new ConfigException("@" + RequestParameters.class.getSimpleName() + " cannot be used with native objects (Integer, String, etc). Failing event: " + getEvent().getPrintName());
		}
		super.setConverter(converter);
	}
	
	@Override
	public boolean matches(String propertyPath) {
		return true;
		//return getPropertyBinder(PropertyUtils.normalize(propertyPath)) != null;
	}
	
	@Override
	public String getDisplayName() {
		return getClass().getSimpleName();
	}
	
}
