package org.loom.mapping;

import java.text.ParsePosition;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang.StringUtils;
import org.loom.exception.ConfigException;
import org.loom.exception.ParameterException;
import org.loom.mapping.UriToken.ParserAction;

import com.google.common.collect.ListMultimap;
import com.google.common.collect.Maps;

/**
 * Parses an incoming request and extracts any parameter values in its path.
 * The expected format is (handle(\{param1(\?)?})?)+
 * Example: /entries/{param1}/{param2?} where param1 is required and param2 is not
 * @author icoloma
 */
public class UriParser {

	/** the list of RequestPathToken to parse */
	private List<UriToken> tokens = new ArrayList<UriToken>();
	
	/** the set of parameter names that are used by this parser */
	private Set<String> parameterNames = new TreeSet<String>();
	
	/** the pattern of uri parameters */
	private static Pattern parameterPattern = Pattern.compile("\\{[^}]+}");
	
	/**
	 * Create a new UriParser based on the uri expression
	 * @param expression the String to be parsed, in the form '/controller/{param1}/{param2}' or similar.
	 * @return the instantiated RequestUriParser
	 */
	public static UriParser newInstance(String expression) {
		UriParser parser = new UriParser();
		Matcher m = parameterPattern.matcher(expression);
		int index = 0;
		int end = 0;
		
		boolean optionalFound = false;
		// extract parameters and handles
		while (m.find() && end < expression.length()) {
			int paramStart = m.start();
			end = m.end();
			UriToken token = new UriToken();
			String handle = expression.substring(index, paramStart);
			if (handle.length() == 0 && index != 0) {
				throw new ConfigException("Parameters must include a separating handle (error at position " + paramStart + " of " + expression + ")");
			}
			token.setHandle(handle);
			String name = expression.substring(paramStart + 1, end - 1);
			
			// if name ends with '?', mark as optional
			int pos = name.length() - 1;
			if (pos > 0 && name.charAt(pos) == '?') {
				optionalFound = true;
				token.setRequired(false);
				name = name.substring(0,  pos);
			} 

			if (name.length() > 0) {
				if (optionalFound && token.isRequired()) {
					throw new ConfigException("Illegal mapping expression '" + expression + "'. Required parameters are not permitted after an optional parameter");
				}
				token.setName(name);
			}
			parser.addToken(token);
			index = end;
		}
		
		// one last handle, without parameter
		if (end < expression.length()) {
			UriToken token = new UriToken();
			String handle = expression.substring(end);
			token.setHandle(handle);
			parser.addToken(token);
		}
		return parser;
	}
	
	/**
	 * Parse the provided URI and return a list of the parameters that were found.
	 * @param uri the uri to be parsed
	 * @return the parsed list of parameters, null if the provided uri was not a match.
	 */
	public Map<String, String> parse(String uri) {
		Map<String, String> params = Maps.newLinkedHashMap();
		
		// parse tokens
		ParsePosition pp = new ParsePosition(0);
		if (tokens.size() > 0) {
			UriToken t1 = tokens.get(0);
			for (int tokenIndex = 1; tokenIndex <= tokens.size(); tokenIndex++) {
				UriToken t2 = tokenIndex == tokens.size()? null : tokens.get(tokenIndex);
				ParserAction action = t1.match(uri, pp, params, t2);
				if (action == ParserAction.FAIL) {
					return null;
				} else if (action == ParserAction.END) {
					break;
				}
				t1 = t2;
			}
		}
		
		// if there is still uri to be parsed, it was not a match
		int parseIndex = pp.getIndex();
		return (parseIndex == uri.length() || parseIndex == uri.length() - 1 && uri.charAt(parseIndex) == '/')? params : null;
	}
	
	public void addToken(UriToken token) {
		tokens.add(token);
		if (token.getName() != null) {
			parameterNames.add(token.getName());
		}
	}
	
	@Override
	public String toString() {
		StringBuilder b = new StringBuilder();
		for (UriToken token : tokens) {
			b.append(token.getHandle());
			if (token.getName() != null) {
				b.append('{').append(token.getName());
				if (!token.isRequired()) {
					b.append('?');
				}
				b.append('}');
			}
		}
		return b.toString();
	}

	public List<UriToken> getTokens() {
		return tokens;
	}

	/**
	 * Produce a String which is this parser representation with the provided parameters
	 * @param params the list of parameters to be included
	 * @return the String URI with the provided params
	 * @throws IllegalArgumentException if any required parameter is missing or there is 
	 * more than one value for the same parameter 
	 */
	public String format(ListMultimap<String, String> params) {
		StringBuilder b = new StringBuilder();
		format(b, params);
		return b.toString();
	}
	public void format(StringBuilder builder, ListMultimap<String, String> params) {
		for (UriToken token : tokens) {
			builder.append(token.getHandle()); 
			if (token.getName() != null) {
				List<String> values = params.get(token.getName());
				if (!values.isEmpty()) {
					if (values.size() > 1) {
						throw new ParameterException("Cannot build " + this + ": parameter '" + token.getName() + "' has multiple values " + StringUtils.join(values, ", "));
					}
					builder.append(values.get(0));
				} else if (token.isRequired()) {
					throw new ParameterException("Cannot build " + this + ": missing required parameter '" + token.getName() + "'");
				} else {
					break;
				}
			}
		}
	}

	public Set<String> getParameterNames() {
		return parameterNames;
	}

	/**
	 * @return true if this UriParser can produce the same URI as the provided one
	 */
	public boolean overlaps(UriParser uriParser) {
		if (tokens.size() != uriParser.getTokens().size()) {
			return false;
		}
		List<UriToken> l1 = tokens;
		List<UriToken> l2 = uriParser.getTokens();
		for (int i = 0; i < l1.size(); i++) {
			UriToken t1 = l1.get(i);
			UriToken t2 = l2.get(i);
			if (!t1.getHandle().equals(t2.getHandle()) || 
					t1.getName() != null && t1.isRequired() && t2.getName() == null || 
					t1.getName() == null && t2.isRequired() && t2.getName() != null) {
				return false;
			}
		}
		
		return true;
	}
	
}
