package org.loom.persistence;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.FlushModeType;
import javax.persistence.LockModeType;
import javax.persistence.Query;

import org.apache.commons.lang.StringUtils;
import org.loom.exception.EntityNotFoundException;
import org.loom.paged.PagedListCriteria;
import org.loom.paged.PagedListData;
import org.loom.paged.PagedListDataFactory;
import org.loom.paged.PagedListDataImpl;

/**
 * Default implementation of the extended operations
 * @author icoloma
 *
 */
@SuppressWarnings("unchecked")
public abstract class AbstractExtendedEntityManager implements ExtendedEntityManager {

	private EntityManager entityManager;
	
	public AbstractExtendedEntityManager(EntityManager entityManager) {
		this.entityManager = entityManager;
	}
	
	public <T> PagedListData<T> query(PagedListCriteria criteria) {
    	
    	// execute the query
    	String q = criteria.getOrderedQuery();
		Query query = entityManager.createQuery(q);
    	query.setFirstResult(criteria.getRecordIndex());
    	query.setMaxResults(criteria.getPageSize());
    	setParameters(query, criteria.getParameters().toArray());
    	
    	PagedListDataImpl<T> result = (PagedListDataImpl<T>) PagedListDataFactory.create(criteria);
    	result.setData(query.getResultList());
    	
    	// obtains the total query results. delete possible "select" and "order by" clauses
    	if (criteria.calculateTotal()) {
	    	Query count = entityManager.createQuery("select count(*) from " + StringUtils.substringAfter(StringUtils.substringBefore(q, "order by"), "from"));
	    	setParameters(count, criteria.getParameters().toArray());
	    	result.setTotalRecords(((Number)count.getSingleResult()).intValue());
    	}
    	return result;
    	
    }

	public <T> List<T> find(String queryString, Object... params) {
		Query query = entityManager.createQuery(queryString);
		setParameters(query, params);
		return query.getResultList();
	}
	
	public <T> T findSingle(String queryString, Object... params) {
		Query query = entityManager.createQuery(queryString);
		setParameters(query, params);
		return (T) query.getSingleResult();
	}

	public <T> T remove(Class<T> clazz, Object id) {
		T instance = find(clazz, id);
		entityManager.remove(instance);
		return instance;
	}
	
	public <T> List<T> findAll(Class<T> clazz) {
		return (List<T>)find("select c from " + clazz.getName() + " c");
	}

	public <T> T find(Class<T> clazz, Object id) {
		T result = entityManager.find(clazz, id);
		if (result == null) {
			throw new EntityNotFoundException(clazz, id);
		}
		return result;
	}
	
	public <T> List<T> findNamed(String namedQuery, Object... params) {
		Query query = entityManager.createNamedQuery(namedQuery);
		setParameters(query, params);
		return query.getResultList();
	}
	
	public <T> T findNamedSingle(String namedQuery, Object... params) {
		Query query = entityManager.createNamedQuery(namedQuery);
		setParameters(query, params);
		return (T) query.getSingleResult();
	}
	
	public int update(String queryString, Object... params) {
		Query query = entityManager.createQuery(queryString);
		setParameters(query, params);
		return query.executeUpdate();
	}

	private void setParameters(Query query, Object... params) {
		if (params != null) {
			for (int i = 0; i < params.length; i++) {
				query.setParameter(i + 1, params[i]);
			}
		}
	}
	
	public void remove(Object entity) {
		getEntityManager().remove(entity);
	}

	public void clear() {
		getEntityManager().clear();
	}

	public void close() {
		getEntityManager().close();
	}

	public boolean contains(Object entity) {
		return getEntityManager().contains(entity);
	}

	public Query createNamedQuery(String name) {
		return getEntityManager().createNamedQuery(name);
	}

	public Query createNativeQuery(String sqlString, Class resultClass) {
		return getEntityManager().createNativeQuery(sqlString, resultClass);
	}

	public Query createNativeQuery(String sqlString, String resultSetMapping) {
		return getEntityManager().createNativeQuery(sqlString, resultSetMapping);
	}

	public Query createNativeQuery(String sqlString) {
		return getEntityManager().createNativeQuery(sqlString);
	}

	public Query createQuery(String qlString) {
		return getEntityManager().createQuery(qlString);
	}

	public void flush() {
		getEntityManager().flush();
	}

	public Object getDelegate() {
		return getEntityManager().getDelegate();
	}

	public FlushModeType getFlushMode() {
		return getEntityManager().getFlushMode();
	}

	public <T> T getReference(Class<T> entityClass, Object primaryKey) {
		return getEntityManager().getReference(entityClass, primaryKey);
	}

	public EntityTransaction getTransaction() {
		return getEntityManager().getTransaction();
	}

	public boolean isOpen() {
		return getEntityManager().isOpen();
	}

	public void joinTransaction() {
		getEntityManager().joinTransaction();
	}

	public void lock(Object entity, LockModeType lockMode) {
		getEntityManager().lock(entity, lockMode);
	}

	public <T> T merge(T entity) {
		return getEntityManager().merge(entity);
	}

	public void persist(Object entity) {
		getEntityManager().persist(entity);
	}

	public void refresh(Object entity) {
		getEntityManager().refresh(entity);
	}

	public void setFlushMode(FlushModeType flushMode) {
		getEntityManager().setFlushMode(flushMode);
	}

	public EntityManager getEntityManager() {
		return entityManager;
	}


}
