/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.persistence.file;

import java.io.InputStream;
import java.security.DigestInputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.sql.Types;

import javax.inject.Inject;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.sql.DataSource;

import org.loom.exception.EntityNotFoundException;
import org.loom.persistence.ExtendedEntityManager;
import org.loom.servlet.params.FileParameter;
import org.loom.util.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.support.SqlLobValue;
import org.springframework.jdbc.support.lob.DefaultLobHandler;
import org.springframework.jdbc.support.lob.LobHandler;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Handles file upload and download
 * 
 * @author icoloma
 */
@Service
public class DatabaseFileManager implements FileManager {

	/** the JPA entity manager */
	@PersistenceContext
	private ExtendedEntityManager entityManager;

	/** handles the LOB creation. For details, see http://icoloma.blogspot.com/2006/11/how-to-handle-oracle-blobs-inside-app.html */
	private LobHandler lobHandler = new DefaultLobHandler();

	/** name of the JDBC table */
	private String fileTableName = "file_contents";

	/** name of the JDBC column */
	private String contentsColumnName = "contents";

	private JdbcTemplate jdbcTemplate = new JdbcTemplate();
	
	@Transactional
	public PersistentFile merge(FileParameter parameter) {
		return merge(new PersistentFile(parameter));
	}
	
	@Transactional
	public PersistentFile merge(PersistentFile file) {
		
		try {
			// merge all  file attributes but contents
			PersistentFile persisted = entityManager.merge(file);
			
			// merge contents
			MessageDigest digest = MessageDigest.getInstance("MD5");
			InputStream input = new DigestInputStream(file.getContents().getContentsAsStream(), digest);
			BlobPersistentFileContents contents = saveContents(persisted,  input, file.getId() != null);
			persisted.setContents(contents);
			persisted.setMD5(StringUtils.toString(digest));
			return persisted;
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Persist the file contents into the database
	 * @param file The PersistentFile instance
	 * @param update true to update an already existing file, false to insert a new one
	 */
	protected BlobPersistentFileContents saveContents(PersistentFile file, InputStream input, boolean update) {
		BlobPersistentFileContents contents;
		if (update) {
			contents = findContents(file.getId());
		} else {
			contents = new BlobPersistentFileContents();
			contents.setFile(file);
			entityManager.persist(contents);
			entityManager.flush();
		}
		String query = "update " + fileTableName + " set " + contentsColumnName + "=? where id = ?";
		jdbcTemplate.update(query, 
				new Object[] {
					new SqlLobValue(input, file.getFileSize().intValue(), getLobHandler()),
					contents.getId() 
					}, 
				new int[] { Types.BLOB, Types.INTEGER });
		
		// returns the uploaded file id 
		entityManager.refresh(contents);
		return contents;
	}

	public PersistentFile find(Integer id) {
		PersistentFile file = entityManager.find(PersistentFile.class, id);
		if (file == null) {
			throw new EntityNotFoundException(PersistentFile.class, id);
		}
		BlobPersistentFileContents contents = findContents(id);
		file.setContents(contents);
		return file;
	}

	@Transactional
	public PersistentFile remove(Integer id) {
		BlobPersistentFileContents contents = findContents(id);
		entityManager.remove(BlobPersistentFileContents.class, contents.getId());
		return entityManager.remove(PersistentFile.class, id);
	}
	
	/**
	 * @return the BlobPersistentFileContents corresponding to the provided file ID
	 * @throws EntityNotFoundException if the PersistentFile instance is not found
	 */
	private BlobPersistentFileContents findContents(Integer id) {
		try {
			return (BlobPersistentFileContents) entityManager.findSingle("from BlobPersistentFileContents contents where contents.file.id=?", id);
		} catch (NoResultException e) {
			throw new EntityNotFoundException(PersistentFile.class, id, e);
		}
	}
	
	@Inject
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate.setDataSource(dataSource);
	}

	public void setContentsColumnName(String contentsColumnName) {
		this.contentsColumnName = contentsColumnName;
	}

	public void setFileTableName(String fileTableName) {
		this.fileTableName = fileTableName;
	}

	public void setLobHandler(LobHandler lobHandler) {
		this.lobHandler = lobHandler;
	}

	protected LobHandler getLobHandler() {
		return lobHandler;
	}

	public void setEntityManager(ExtendedEntityManager entityManager) {
		this.entityManager = entityManager;
	}

}
