/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.resolution;


import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;

import org.loom.servlet.CacheControl;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.servlet.names.ResponseHeaderNames;
import org.loom.util.HtmlSanitizer;
import org.loom.util.MimeUtils;

/**
 * Delivers a binary file to the browser
 * @author Ignacio Coloma
 */
public abstract class AbstractFileResolution extends AbstractHttpResolution {

	/** the file name. If content type is not set guesses the mime from the file name. If null, no file name will be provided to the browser */
	private String filename;

	/** the file size */
	private Long fileSize;
	
	/** prompt the user to save the file instead of trying to open it; defaults false */
	private boolean forceSave = false;
	
	/** true to guess the file content type (if null) based on the file extension, defaults true */
	private boolean guessMimeType = true;

	@Override
	public abstract void resolve(LoomServletRequest request, LoomServletResponse response) throws IOException, ServletException;

	/**
	 * Calculate default content size, content type, etc headers that will be applied if 
	 * the user did not provide one previously.
	 */
	@Override
	protected void writeHeaders(LoomServletRequest request, LoomServletResponse response) {
		String filename = getFilename();
		if (getContentType() == null && guessMimeType) {
			setContentType(MimeUtils.guessMimeType(filename));
		}
		
		setHeader(ResponseHeaderNames.CONTENT_DISPOSITION, calculateContentDispositionHeader(filename));
		Long fileSize = getFileSize();
		if (fileSize != null) {
			setHeader(ResponseHeaderNames.CONTENT_LENGTH, fileSize.intValue());
		}
		super.writeHeaders(request, response);
	}
	
	public String calculateContentDispositionHeader(String filename) {
		return filename == null? null : (forceSave? "attachment" : "inline") + "; filename=\"" + HtmlSanitizer.sanitize(filename.replace('"', '\'')) + "\"";
	}

	public AbstractFileResolution setForceSave(boolean forceSave) {
		this.forceSave = forceSave;
		return this;
	}

	public boolean isGuessMimeType() {
		return guessMimeType;
	}

	public AbstractFileResolution setGuessMimeType(boolean guessMimeType) {
		this.guessMimeType = guessMimeType;
		return this;
	}

	public boolean isForceSave() {
		return forceSave;
	}

	public String getFilename() {
		return filename;
	}

	public AbstractFileResolution setFilename(String filename) {
		this.filename = filename;
		return this;
	}

	public Long getFileSize() {
		return fileSize;
	}

	public AbstractFileResolution setFileSize(Long fileSize) {
		this.fileSize = fileSize;
		return this;
	} 
	
	@Override
	public AbstractFileResolution addCookie(String name, String value) {
		return (AbstractFileResolution) super.addCookie(name, value);
	}
	
	@Override
	public AbstractFileResolution addCookie(String name, String value, int maxAge) {
		return (AbstractFileResolution) super.addCookie(name, value, maxAge);
	}
	
	@Override
	public AbstractFileResolution addCookie(Cookie cookie) {
		return (AbstractFileResolution) super.addCookie(cookie);
	}
	
	@Override
	public AbstractFileResolution addHeader(String name, Object value) {
		return (AbstractFileResolution) super.addHeader(name, value);
	}
	
	@Override
	public AbstractFileResolution setHeader(String name, Object value) {
		return (AbstractFileResolution) super.setHeader(name, value);
	}
	
	@Override
	public AbstractFileResolution setHeaderIfNotPresent(String name, Object value) {
		return (AbstractFileResolution) super.setHeaderIfNotPresent(name, value);
	}
	
	@Override
	public AbstractFileResolution setCharset(String charset) {
		return (AbstractFileResolution) super.setCharset(charset);
	}
	
	@Override
	public AbstractFileResolution setContentType(String contentType) {
		return (AbstractFileResolution) super.setContentType(contentType);
	}
	
	@Override
	public AbstractFileResolution setCacheControl(CacheControl cacheControl) {
		return (AbstractFileResolution) super.setCacheControl(cacheControl);
	}
}
