/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.resolution;

import javax.servlet.http.Cookie;

import org.loom.servlet.CacheControl;


/**
 * A response object with http headers. 
 * 
 * @author Ignacio Coloma
 */
public interface HttpResolution extends CacheableResolution {

	/**
	 * Adds a cookie that will not be stored on disk (will be deleted when the browser is closed). 
	 * @param name the name of the cookie
	 * @param value the value of the Cookie
	 */
	public HttpResolution addCookie(String name, String value);
	
	/**
	 * Adds a cookie  
	 * @param name the name of the cookie
	 * @param value the value of the Cookie
	 * @param maxAge
	 *            can be positive (number of seconds that it will be stored in
	 *            the browser), zero (to delete the cookie from the browser) or
	 *            a negative value (to indicate that the cookie should not be
	 *            stored in disk and will be forgotten when the browser exists).
	 */
	public HttpResolution addCookie(String name, String value, int maxAge);
	
	/**
	 * Adds a cookie
	 * @param cookie the cookie to be added to the response
	 */
	public HttpResolution addCookie(Cookie cookie);
	
	/**
	 * Add a http header. If the header already exists a new value will be added.
	 * @param name the name of the http header
	 * @param value the value of the http header. Date, int and String are supported. Anything else  will be cast to toString(). Null will be ignored.
	 */
	public HttpResolution addHeader(String name, Object value);
	

	/**
	 * Set a String http header. If the header already exists it will be replaced.
	 * @param name the name of the http header
	 * @param value the value of the http header
	 */
	public HttpResolution setHeader(String name, Object value);
	
	/**
	 * Set a String http header if it has not been set.
	 * @param name the name of the http header
	 * @param value the value of the http header
	 */
	public HttpResolution setHeaderIfNotPresent(String name, Object value);
	
	/**
     * Sets the character encoding (MIME charset) of the response
     * being sent to the client, for example, to UTF-8.
     */
	public HttpResolution setCharset(String charset);
	
	/**
     * Sets the content type of the response being sent to
     * the client, if the response has not been committed yet.
     * The given content type may include a character encoding
     * specification, for example, <code>text/html;charset=UTF-8</code>.
     * Notice that #setCharset(String) can also be used for this.
     */
	public HttpResolution setContentType(String contentType);
	
	public HttpResolution setCacheControl(CacheControl cacheControl);
	
}