/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.resolution;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.servlet.ServletException;

import org.apache.commons.io.IOUtils;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.util.FileUtils;

/**
 * Deliver the contents from an InputStream to the browser. 
 * The stream will be closed when finished.
 * @author Rafael Serrano
 */
public class InputStreamResolution extends AbstractFileResolution {

	/** the stream to read from */
	private InputStream stream;

	/**
	 * Create a File-based InputStreamResolution. 
	 * @param file the file to be delivered to the browser
	 * @throws FileNotFoundException if the file could not be found
	 */
	public InputStreamResolution(File file) throws FileNotFoundException {
		this(file.getName(), file);
	}
	
	/**
	 * Create a File-based InputStreamResolution
	 * @param filename the file name that will be used by the browser in the "Save-as" dialog
	 * @param file the file to be delivered to the browser
	 * @throws FileNotFoundException if the file could not be found
	 */
	public InputStreamResolution(String filename, File file) throws FileNotFoundException {
		this(filename, new FileInputStream(file), file.length());
	}
	
	/**
	 * Create a InputStreamResolution based on an InputStream
	 * Note that this construtor will not hint the browser about the saved file name.
	 * @param stream the inputStream to read contents from
	 */
	public InputStreamResolution(InputStream stream) {
		this(null, stream, null);
	}
	
	/**
	 * Create a InputStreamResolution based on an InputStream
	 * Note that this construtor will not hint the browser about the saved file name.
	 * @param stream the inputStream to read contents from
	 * @param filename the file name that will be used by the browser in the "Save-as" dialog (may be null)
	 */
	public InputStreamResolution(String filename, InputStream stream) {
		this(filename, stream, null);
	}
	
	/**
	 * Create a InputStreamResolution based on an InputStream
	 * Note that this construtor will not hint the browser about the saved file name.
	 * @param stream the inputStream to read contents from
	 * @param filename the file name that will be used by the browser in the "Save-as" dialog (may be null)
	 * @param fileSize the file size (null if unknown)
	 */
	public InputStreamResolution(String filename, InputStream stream, Long fileSize) {
		if (stream == null) {
			throw new IllegalArgumentException("stream cannot be null");
		}
		this.stream = stream;
		this.setFilename(filename);
		this.setFileSize(fileSize);
	}

	public void resolve(LoomServletRequest request, LoomServletResponse response)
			throws IOException, ServletException {
		try {
			writeHeaders(request, response);
			FileUtils.copy(stream, response.getOutputStream());
		} finally {
			IOUtils.closeQuietly(stream);
		}
	}

}
