/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.resolution;

import java.util.List;

import javax.servlet.http.Cookie;

import org.loom.servlet.CacheControl;


/**
 * An interface implemented by redirect resolutions.
 * 
 * @author icoloma
 */
public interface RedirectResolution extends Resolution, HttpResolution {

	/**
	 * Add a parameter to the redirected url
	 * @param name the name of the parameter
	 * @param value the value of the parameter
	 * @return This same instance
	 * 
	 */
	public RedirectResolution add(String name, Object value);

	/**
	 * Add a parameter to the redirected url
	 * @param name the name of the parameter
	 * @param value the value of the parameter
	 * @return This same instance
	 * @deprecated use {@link RedirectResolution#add(String, Object)} instead 
	 */
	public RedirectResolution addParameter(String name, Object value);

	/**
	 * Replace a parameter value. If the parameter is multiple, all of them will be 
	 * removed and a single value will be inserted instead. 
	 * @param name the name of the parameter to be replaced.
	 * @param value the new value of the parameter. If null, the parameter will just be removed.
	 */
	public RedirectResolution replace(String name, Object value);
	
	/**
	 * Replace a parameter value. If the parameter is multiple, all of them will be 
	 * removed and a single value will be inserted instead. 
	 * @param name the name of the parameter to be replaced.
	 * @param value the new value of the parameter. If null, the parameter will just be removed.
	 * @deprecated use {@link RedirectResolution#replace(String, Object)} instead
	 */
	public RedirectResolution replaceParameter(String name, Object value);

	/**
	 * Remove a GET parameter
	 * @param name the name of the Parameter to remove
	 * @return the list of removed values
	 */
	public List<String> remove(String name);
	
	/**
	 * Remove a GET parameter
	 * @param name the name of the Parameter to remove
	 * @return the list of removed values
	 * @deprecated use {@link RedirectResolution#remove(String)} instead
	 */
	public List<String> removeParameter(String name);

	/**
	 * Add an action property as a parameter.
	 * The property value will be retrieved from the current action and its name
	 * and value appended to the resulting URL.
	 * @param propertyName the name of the action property to be added. 
	 * @return This same instance
	 */
	public RedirectResolution addProperty(String propertyName);
	
	/**
	 * Add an action property as a parameter.
	 * The property value will be retrieved from the current action and its name
	 * and value appended to the resulting URL.
	 * @param propertyName the name of the action property to be added. 
	 * @return This same instance
	 * @deprecated use {@link RedirectResolution#addProperty(String)} instead
	 */
	public RedirectResolution addPropertyParameter(String propertyName);
	
	/**
	 * Add a hash to the URL. A hash is the URL part after the '#' 
	 * @param hash The hash to be added
	 * @return This same instance
	 */
	public RedirectResolution setHash(String hash);

	/** 
	 * Set to not null to override the request server name
	 */
	public RedirectResolution setHostname(String hostname);
	
	/** 
	 * Set to not null to override the request scheme.
	 * 
	 * @param scheme the scheme name, without colon: "http", "https", etc. 
	 * Use the empty String to generate a protocol-less URL, e.g. "//myhost.com/resource.html"
	 */
	public RedirectResolution setScheme(String scheme);
	
	/** 
	 * Set to not null to override the request port 
	 */
	public RedirectResolution setPort(Integer port);

	/** 
	 * true to append to the generated URL all current request parameters, default false.
	 * Parameters added to this {@link RedirectResolution} instance will override the value included in the request.
	 */
	public RedirectResolution setIncludeRequestParameters(boolean addCurrentRequestParameters);
	
	/**
	 * Invoke this method to perform a permanent redirect (301) instead of a temporary redirect (302).
	 */
	public RedirectResolution permanent();
	
	public RedirectResolution addCookie(String name, String value);
	
	public RedirectResolution addCookie(String name, String value, int maxAge);
	
	public RedirectResolution addCookie(Cookie cookie);
	
	public RedirectResolution addHeader(String name, Object value);
	
	public RedirectResolution setHeader(String name, Object value);
	
	public RedirectResolution setHeaderIfNotPresent(String name, Object value);
	
	public RedirectResolution setCharset(String charset);
	
	public RedirectResolution setContentType(String contentType);

	public RedirectResolution setCacheControl(CacheControl cacheControl);
	
}
