package org.loom.servlet;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.Collection;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.inject.Inject;
import javax.inject.Singleton;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.fileupload.FileItemIterator;
import org.apache.commons.fileupload.FileItemStream;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.IOUtils;
import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;
import org.loom.config.LocaleResolver;
import org.loom.i18n.MessagesRepository;
import org.loom.i18n.MessagesRepositoryFactory;
import org.loom.mapping.ActionMappingRepository;
import org.loom.servlet.names.RequestAttributeNames;
import org.loom.servlet.params.FileParameterFactory;
import org.loom.servlet.params.ServletRequestParameters;
import org.loom.servlet.scope.FlashContextFactory;
import org.loom.util.StringUtils;

import com.google.common.collect.Lists;

@Singleton
public class LoomServletRequestFactory {
	
	@Inject
	private ServletContext servletContext;
	
	/**
	 * creates MessagesRepository instances with the locale-aware contents adapted to the
	 * current request locale
	 */
	@Inject
	private MessagesRepositoryFactory messagesRepositoryFactory;
	
	/**
	 * calculates the Locale to use for the current request
	 */
	@Inject
	private LocaleResolver localeResolver;
	
	/**
	 * Creates a flash context for the current request
	 */
	@Inject
	private FlashContextFactory flashContextFactory;

	/** creates uploaded file items (adapter for commons-fileupload) */
	@Inject
	private FileParameterFactory fileParameterFactory;
	
	/** parses parameters from application/json requests */
	private ObjectMapper objectMapper = new ObjectMapper();

	@Inject 
	private ActionMappingRepository actionMappingRepository;
	
	/** the default charset to be used for text/*, application/x-json and application/x-javascript responses, defaults to UTF-8 */
	private String defaultCharset = "UTF-8";
	
	public LoomServletRequest getOrCreateInstance(ServletRequest parent) {
		LoomServletRequest instance = (LoomServletRequest) parent.getAttribute(RequestAttributeNames.REQUEST);
		if (instance == null) {
			instance = createInstance((HttpServletRequest) parent);
		}
		return instance;
	}
	
	public void clearInstance(ServletRequest request) {
		request.removeAttribute(RequestAttributeNames.REQUEST);
		LoomServletRequestImpl.clearThreadLocal();
	}

	private LoomServletRequest createInstance(HttpServletRequest request) {
		LoomServletRequestData data = new LoomServletRequestData();
		data.setDefaultCharset(defaultCharset);
		data.setServletContext(servletContext);
		data.setMessagesRepository(getMessagesRepository(request));
		data.setFlashContextFactory(flashContextFactory);
		data.setServletRequestParameters(createServletRequestParameters(request));
		data.setActionMappingRepository(actionMappingRepository);
		
		LoomServletRequestImpl instance = new LoomServletRequestImpl(request, data);
		request.setAttribute(RequestAttributeNames.REQUEST, instance);
		LoomServletRequestImpl.setThreadLocal(instance);
		return instance;
	}

	@SuppressWarnings("unchecked")
	private ServletRequestParameters createServletRequestParameters(HttpServletRequest request) {
		if (request.getCharacterEncoding() == null && defaultCharset != null) {
			try {
				request.setCharacterEncoding(defaultCharset);
			} catch (UnsupportedEncodingException e) {
				throw new RuntimeException(e);
			}
		}
		
		ServletRequestParameters parameters = new ServletRequestParameters();
		if (!"get".equals(request.getMethod().toLowerCase())) {
			if ("application/json".equalsIgnoreCase(request.getContentType())) {
				convertJsonParameters(request, parameters);
			} else if (ServletFileUpload.isMultipartContent(request)) {
				convertMultipartFormParameters(request, parameters);
			}
		}
		
		// add normal parameters. If multipart, this will only add any extra GET parameter (not POST)
		for (Enumeration<String> e = request.getParameterNames(); e.hasMoreElements(); ) {
			String paramName = e.nextElement();
			parameters.addStringParameter(paramName, request.getParameter(paramName));
		}
		return parameters;
	}
	
	/**
	 * Separate multipart form params by type: normal form parameters are
	 * returned as a Map, and the rest is stored in a list of FileItem
	 */
	private void convertMultipartFormParameters(HttpServletRequest request, ServletRequestParameters parameters) {
		try {
			ServletFileUpload fileUpload = new ServletFileUpload();

			// commons-fileupload uses the http default (ISO-8859-1) when the charset is missing, while the browser uses the same 
			// charset of the last response (which should be config.defaultCharset)
			String charset = request.getCharacterEncoding();
			
			for (FileItemIterator iterator = fileUpload.getItemIterator(request); iterator.hasNext(); ) {
				FileItemStream item = iterator.next();
				String name = item.getFieldName();
				if (!parameters.contains(name)) { // avoid overwriting with the last value
					if (item.isFormField()) { // simple field, add as String
						InputStream is = item.openStream();
						try {
							String value = IOUtils.toString(is, charset);
							parameters.addStringParameter(name, value);
						} finally {
							IOUtils.closeQuietly(is);
						}
					} else { 
						parameters.addFileParameter(name, fileParameterFactory.create(item));
					}
				}

			}
			
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		} catch (FileUploadException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	
	/**
	 * Parse a JSON request, converting it into parameters
	 */
	private void convertJsonParameters(HttpServletRequest request, ServletRequestParameters parameters) {
		try {
			Map<String, Object> values = objectMapper.readValue(request.getInputStream(), Map.class);
			List<String> paramNamePath = Lists.newArrayList();
			convertJsonParameters(parameters, paramNamePath, values);
		} catch (JsonParseException e) {
			throw new RuntimeException(e);
		} catch (JsonMappingException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		
	}
	
	private void convertJsonParameters(ServletRequestParameters parameters, List<String> paramNamePath, Map<String, Object> values) {
		for (Map.Entry<String, Object> entry : values.entrySet()) {
			convertJsonParameter(parameters, paramNamePath, entry.getKey(), entry.getValue());
		}
	}

	private void convertJsonParameter(ServletRequestParameters parameters, List<String> paramNamePath, String paramName, Object value) {
		if (value != null) {
			if (value instanceof Collection) {
				int index = 0;
				for (Object item : (Collection) value) {
					convertJsonParameter(parameters, paramNamePath, paramName + "[" + index + "]", item);
					index++;
				}
			} else if (value instanceof Map) {
				paramNamePath.add(paramName);
				convertJsonParameters(parameters, paramNamePath, (Map<String, Object>) value);
				paramNamePath.remove(paramNamePath.size() - 1);
			} else {
				paramNamePath.add(paramName);
				parameters.addStringParameter(StringUtils.join(paramNamePath, "."), value.toString());
				paramNamePath.remove(paramNamePath.size() - 1);
			}
		}
	}

	private MessagesRepository getMessagesRepository(HttpServletRequest request) {
		Locale locale = localeResolver.getLocale(request);
		if (locale == null) {
			locale = request.getLocale();
		}
		return messagesRepositoryFactory.getRepository(locale);
	}
	
	/**
	 * Returns the instance of LoomServletResponse if one already exists, or creates a new one if not
	 */
    public LoomServletResponse getResponseInstance(LoomServletRequest request, ServletResponse parent) {
    	LoomServletResponseImpl response = (LoomServletResponseImpl) request.getAttribute(RequestAttributeNames.RESPONSE);
    	if (response != null) {
    		return response;
    	}

        // set the same Locale that has been calculated for the request object
    	response = new LoomServletResponseImpl(parent, initResponseData(request.getLocale()));
    	request.setAttribute(RequestAttributeNames.RESPONSE, response);
        return response;

    }
    
    private LoomServletResponseData initResponseData(Locale locale) {
    	LoomServletResponseData data = new LoomServletResponseData();
    	data.setCharset(defaultCharset);
    	data.setLocale(locale);
    	return data;
    }
    
	public void setServletContext(ServletContext servletContext) {
		this.servletContext = servletContext;
	}

	public void setMessagesRepositoryFactory(
			MessagesRepositoryFactory messagesRepositoryFactory) {
		this.messagesRepositoryFactory = messagesRepositoryFactory;
	}

	public void setLocaleResolver(LocaleResolver localeResolver) {
		this.localeResolver = localeResolver;
	}

	public void setFlashContextFactory(FlashContextFactory flashContextFactory) {
		this.flashContextFactory = flashContextFactory;
	}

	public void setDefaultCharset(String defaultCharset) {
		this.defaultCharset = defaultCharset;
	}

	public ServletContext getServletContext() {
		return servletContext;
	}

	public MessagesRepositoryFactory getMessagesRepositoryFactory() {
		return messagesRepositoryFactory;
	}

	public LocaleResolver getLocaleResolver() {
		return localeResolver;
	}

	public FlashContextFactory getFlashContextFactory() {
		return flashContextFactory;
	}

	public String getDefaultCharset() {
		return defaultCharset;
	}

	public void setFileParameterFactory(FileParameterFactory fileParameterFactory) {
		this.fileParameterFactory = fileParameterFactory;
	}

	public void setObjectMapper(ObjectMapper objectMapper) {
		this.objectMapper = objectMapper;
	}
	
	/**
	 * Returns the instance of LoomServletRequestWrapper, if one exists, or
	 * instantiates one if not.
	 * If the instance is not found and create is false, throws an exception.
	 * @deprecated use either getInstance or getOrCreateInstance instead
	 * /
	public static LoomServletRequestImpl getInstance(ServletRequest parent, boolean create) {
		return create? getOrCreateInstance(parent) : getInstance(parent);
	}*/

}
