/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.servlet.params;

import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.Scope;
import org.loom.servlet.names.FlashAttributeNames;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.servlet.scope.FlashContext;

import com.google.common.base.Predicate;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;

/**
 * Parameters as received by the request object.
 * This class can only store one value per name. If more than one parameter is added
 * with the same name, only the first one will be processed. To handle multiple values,
 * use indexes (values[2].name) where needed to ensure uniqueness.
 * @author Ignacio Coloma
 */
public class ServletRequestParameters {

	/** String parameter values, uploaded via GET or POST, normal or url-form-encoded */
	private Map<String, String> stringParameters = Maps.newLinkedHashMap();

	/** uploaded file parameters, if any */
	private Map<String, FileParameter> fileParameters = Maps.newLinkedHashMap();
	
	/** set of parameters that have already been assigned */
	private Set<String> assignedParameters = Sets.newHashSet();
	
	public ServletRequestParameters() {
	}
	
	/**
	 * Adds an uploaded file item to the list of parameters
	 * @param file
	 */
	public void addFileParameter(String name, FileParameter file) {
		if (file.getFileSize() == 0) {
			return;
		}
		fileParameters.put(name, file);
	}
	
	/**
	 * Add a parameter value to the list of processed params
	 */
	public void addStringParameter(String paramName, String value) {
		stringParameters.put(paramName, value);
	}
	
	/**
	 * Adds all the provided params to this RequestParams instance
	 * @param params the Map of params to be added
	 */
	public void addStringParameters(Map<String, String> params) {
		this.stringParameters.putAll(params);
	}
	
	/**
	 * Adds all the provided params to this RequestParams instance
	 * @param params the Map of params to be added
	 */
	public void addFileParameters(Map<String, FileParameter> params) {
		this.fileParameters.putAll(params);
	}
	
	public Set<Map.Entry<String, String>> getStringParametersEntrySet() {
		return stringParameters.entrySet();
	}
	
	public Set<Map.Entry<String, FileParameter>> getFileParametersEntrySet() {
		return fileParameters.entrySet();
	}
	
	public String getStringParameter(String name) {
		return stringParameters.get(name);
	}
	
	public FileParameter getFileParameter(String name) {
		return fileParameters.get(name);
	}

	public boolean contains(String paramName) {
		return stringParameters.containsKey(paramName) || fileParameters.containsKey(paramName);
	}
	
	public int size() {
		return stringParameters.size() + fileParameters.size();
	}
	
	/**
	 * @return a list of request params in the form name=value
	 */
	@Override
	public String toString() {
		return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
			.append("stringParameters", stringParameters)
			.append("fileParameters", fileParameters.keySet())
			.append("assignedParameters", assignedParameters).toString();
	}

	/**
	 * @return the string parameters received
	 */
	public Map<String, String> toParameterMap() {
		return stringParameters;
	}
	
	public Set<String> getFileParameterNames() {
		return fileParameters.keySet();
	}

	/**
	 * Set the provided parameter name  as assigned
	 */
	public void setAssigned(String name) {
		assignedParameters.add(name);
	}

	/**
	 * Return true if the provided parameter has already been assigned to a java counterpart (action attribute or event parameter)
	 */
	public boolean isAssigned(String paramName) {
		return assignedParameters.contains(paramName);
	}

	/**
	 * Save the string parameters in this object to the flash context, to be retored at the next request.
	 */
	public void saveForRedirect(LoomServletRequest request) {
		if (!stringParameters.isEmpty()) {
			request.setScopedAttribute(FlashAttributeNames.SAVED_PARAMETERS, stringParameters, Scope.FLASH);
		}
	}

	/**
	 * restores all parameters from a previous request (after a redirect).
	 * This method is intended to restore parameters after an unsuccessful event execution because of errors in the user input data.
	 * @param request
	 */
	@SuppressWarnings("unchecked")
	public void restoreFromRedirect(LoomServletRequest request) {
		FlashContext context = request.getPreviousFlashContext();
		if (context != null) {
			Map<String, String> restoredParams = (Map<String, String>) context.getAttribute(FlashAttributeNames.SAVED_PARAMETERS);
			if (restoredParams != null) {
				// Internal Loom parameters such as OVERRIDE_EVENT must not be restored
				Map<String, String> filteredParams = Maps.filterKeys(restoredParams, new Predicate<String>() {

					@Override
					public boolean apply(String input) {
						return !input.startsWith(RequestParameterNames.LOOM_PREFIX);
					}
					
				});
				stringParameters.putAll(filteredParams);
			}
		}
	}

}
