/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags;

import java.io.IOException;

import javax.inject.Inject;
import javax.servlet.jsp.JspException;

import org.loom.tags.decorator.TagDecorator;
import org.loom.tags.decorator.TagDecoratorRepository;
import org.tldgen.annotations.Attribute;


/**
 * Parent class of any decorated tag
 * @author icoloma
 */
public abstract class AbstractDecoratedTag extends AbstractTag {

	@Inject 
	private TagDecoratorRepository tagDecoratorRepository;
	
	/**
	 * The name of the decorator to use. The default decorators for any form field 
	 * will display the surrounding label. 
	 */
	@Attribute
	protected String decorator;

	/**
	 * Initializes the Request and PageData properties, and forward the call to the child class
	 */
	@Override
	@SuppressWarnings("unchecked")
	public final void doTag() throws JspException, IOException {
		if (renderTag()) {
			initTag();
			try {
				TagDecorator decorator = getDecoratorInstance();
				if (decorator == null || decorator.beforeDoTag(this)) {
					doTagImpl();
				}
				if (decorator != null) {
					decorator.afterDoTag(this);
				}
			} finally {
				endTag();
			}
		}
	}
	
	/**
	 * This is an method intended for component extension. If false, the entire tag 
	 * execution will be skipped, including decorator invocation
	 * @return true to execute the tag, default false.
	 */
	protected boolean renderTag() {
		return true;
	}

	/**
	 * Retrieves the decorator instance. 
	 */
	@SuppressWarnings("unchecked")
	public TagDecorator getDecoratorInstance() {
		if ("none".equals(decorator)) {
			return null;
		}
		return decorator != null? tagDecoratorRepository.getDecorator(decorator) : tagDecoratorRepository.getDecorator(getClass());
	}

	public void setDecorator(String decorator) {
		this.decorator = decorator;
	}

	public String getDecorator() {
		return decorator;
	}


}
