/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags;

import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Extra attributes of a JSP tag. Any attribute that is not specifically handled 
 * by a JSP tag gets stored here. 
 * Attributes will be written in the same order they were added.
 * 
 * @author icoloma
 */
public class TagAttributes {
	
	private Map<String, Object> contents = new LinkedHashMap<String, Object>();

	/** 
	 * prefix applied to all "extra" attributes. We have settled for 
	 * <a href="http://www.w3.org/html/wg/html5/#custom-data-attribute">the "data-" prefix</a> in the HTML 5 spec.
	 * Replace with "l:" or similar to use XHTML namespaces
	 */
	private String prefix = "data-";
	
	/**
	 * Set an attribute if is value is != null
	 * @param name the name of the attribute, without prefix
	 */
	public TagAttributes set(String name, Object value) {
		if (value != null) {
			contents.put(name, value);
		}
		return this;
	}
	
	/**
	 * Write the list of attributes to the output.
	 * Each attribute name get preceded with the prefix value, which is "data-" by default
	 * (the standard recommended by HTML 5). Other alternatives such as namespaced attributes 
	 * are possible.
	 */
	public void print(WriterWrapper<?> out) throws IOException {
		for (Map.Entry<String, Object> attribute : contents.entrySet()) {
			out.printAttribute(prefix + attribute.getKey(), attribute.getValue());
		}
	}
	
	/**
	 * Retrieve an attribute.
	 * @param name the name of the attribute, without the applied prefix
	 */
	public Object get(String name) {
		return contents.get(name);
	}
	
	public void remove(String name) {
		contents.remove(name);
	}
	
	public void clear() {
		contents.clear();
	}

	public String getPrefix() {
		return prefix;
	}

	public void setPrefix(String prefix) {
		this.prefix = prefix;
	}

	public boolean isEmpty() {
		return contents.isEmpty();
	}

	public int size() {
		return contents.size();
	}
	
	@Override
	public String toString() {
		return contents.toString();
	}
	
}
