/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;
import java.util.regex.Pattern;

import javax.servlet.jsp.JspException;

import org.loom.servlet.names.HtmlExtendedAttributeNames;
import org.loom.tags.AbstractFormInputTag;
import org.loom.util.HtmlSanitizer;
import org.tldgen.annotations.Attribute;


/**
 * Parent of the InputText and InputTextArea fields
 * @author icoloma
 *
 */
public abstract class AbstractTextFieldTag extends AbstractFormInputTag {
	
	/** 
	 * This is a known attribute for InputTextField and subclasses, as specified 
	 * <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-maxlength">here</a>. 
	 * In the case of <a href="http://www.whatwg.org/specs/web-forms/current-work/#maxlength">pre-HTML5</a> 
	 * InputTextArea, it should be enforced using javascript. 
	 */
	@Attribute
	protected Integer maxlength;
	
	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-readonly">HTML attribute</a> */
	@Attribute
	protected Boolean readonly;
	
	/** 
	 * <a href="http://www.whatwg.org/specs/web-forms/current-work/#the-pattern">Regex pattern</a>
	 * <strong>HTML 5</strong>. Note that we will be using the regular expression as introduced by the 
	 * developer, not modifying it as the spec requires us to. 
	 */
	@Attribute
	protected String pattern;
	
	/**
	 * Prints this instance attributes
	 */
	@Override
	protected void printParentAttributes() throws IOException, JspException {
		super.printParentAttributes();
		// we are not really protecting against XSS, but regex may contain quotes
		out.printAttribute("pattern", HtmlSanitizer.sanitize(pattern));
		out.printAttribute("readonly", readonly);
		out.printAttribute("maxlength", maxlength);
	}

	public void setMaxlength(Integer maxLength) {
		this.maxlength = maxLength;
	}

	public void setReadonly(Boolean readonly) {
		this.readonly = readonly;
	}

	public String getPattern() {
		return pattern;
	}

	public void setPattern(String pattern) {
		this.pattern = pattern;
	}

	public void setPattern(Pattern maskPattern) {
		if (maskPattern != null) {
			setPattern(maskPattern.pattern());
		}
	}

	public void setMinlength(Integer minLength) {
		setExtendedAttribute(HtmlExtendedAttributeNames.MIN_LENGTH, minLength);
	}

	public Integer getMaxlength() {
		return maxlength;
	}

}
