/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.mapping.ParsedAction;
import org.loom.tags.AbstractTag;
import org.loom.util.HtmlSanitizer;
import org.loom.util.PropertyUtils;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * <a href="http://www.w3.org/TR/html4/interact/forms.html#h-17.4">HTML input hidden tag</a>
 * @author icoloma
 */
@Tag(
		dynamicAttributes=true, 
		bodyContent=BodyContent.EMPTY,
		example="&lt;l:inputHidden name=\"mortgage.id\"/>"
	)
public class InputHiddenTag extends AbstractTag {
	
	/** <a href="http://www.w3.org/TR/html4/struct/global.html#adef-id">HTML attribute</a> */
	@Attribute
	private String id;
	
	private String cssClass;
	
	/** <a href="http://www.w3.org/TR/html4/struct/dirlang.html#adef-lang">HTML attribute</a> */
	@Attribute
	private String lang;

	/** the property path of this field */
	@Attribute(required=true)
	private String name;
	
	/** <a href="http://www.w3.org/TR/html4/present/styles.html#adef-style">HTML attribute</a> */
	@Attribute
	private String style;
	
	/** when specified, use this value instead of the action property value */
	@Attribute
	private String value;
	
	/** true to execute this tag, default true */
	private boolean ifClause = true;

	@Override
	public void doTagImpl() throws JspException, IOException {
		if (ifClause) {
			out.print("<input type=\"hidden\"");
			out.printAttribute("id", id);
			out.printAttribute("class", cssClass);
			out.printAttribute("lang", lang);
			out.printAttribute("name", name);
			out.printAttribute("style", style);
			out.printAttribute("value", HtmlSanitizer.sanitize(getValue()));
			out.print("/>");
		}
	}
	
	/**
	 * @return the value extracted from the nested property inside the action bean
	 */
	public String getValue() {
		if (value != null) {
			return value;
		}
		
		// calculate the value only if the target action is the same as the parsed action
		ParsedAction parsedAction = request.getParsedAction();
		if (parsedAction != null && parsedAction.getActionMapping().getPropertyBinderMap().getPropertyBinder(PropertyUtils.normalize(name)) != null) {
			return parsedAction.getPropertyAsString(getName());
		} else {
			return request.getStringParameter(getName());
		}

	}

	public void setId(String id) {
		this.id = id;
	}

	public void setLang(String lang) {
		this.lang = lang;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setStyle(String style) {
		this.style = style;
	}

	public void setValue(String value) {
		this.value = value;
	}

	public String getName() {
		return name;
	}

	/**
	 * <a href="http://www.w3.org/TR/html4/struct/global.html#adef-class">HTML attribute</a>
	 */
	@Attribute(name="class")
	public void setCssClass(String cssClass) {
		this.cssClass = cssClass;
	}

	/**
	 * Set to false to skip tag execution.
	 */
	@Attribute
	public void setIf(boolean ifClause) {
		this.ifClause = ifClause;
	}

}
