/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.mapping.ParsedAction;
import org.loom.tags.AbstractTag;
import org.loom.url.ParameterContainer;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;


/**
 * A parameter (name/value pair) to be added to the URL created by a ParameterContainer.
 * Examples of ParameterContainer are url, tab, form and almost anything that 
 * can generate a link.
 * <p>
 * It's usually just easier to use a param-[paramName]=[paramValue] in the container tag.
 * </p> 
 * @author icoloma
 */
@Tag(
		dynamicAttributes=true,
		bodyContent=BodyContent.EMPTY,
		example=
		"&lt;l:url action=\"BlogEntries\" event=\"open\">\n" +
		"   &lt;l:param name=\"id\" value=\"1\"/>\n" +
		"   &lt;l:param property=\"entity.id\"/>\n" +
		"&lt;/l:url>\n" +
		"&lt;l:url action=\"BlogEntries\" event=\"open\" param-id=\"1\"/>\n"
	)
public class ParamTag extends AbstractTag {

	/** the name of the parameter */
	@Attribute
	private String name;
	
	/** the value of the parameter, if specified */
	@Attribute
	private String value;
	
	/** 
	 * if specified, <code>name</code> will default to the property name and 
	 * <code>value</code> will be pulled from the specified action property 
	 */
	@Attribute
	private String property;
	
	@Override
	public void doTagImpl() throws JspException, IOException {
		ParameterContainer container = getAncestorWithClass(ParameterContainer.class);
		container.addParameter(getName(), getValue());
	}
	
	protected String getName() {
		if (name != null) {
			return name;
		}
		if (property != null) {
			return property;
		}
		throw new IllegalArgumentException("No name or property value specified for param tag");
	}
	
	protected Object getValue() {
		if (property != null) {
			ParsedAction parsedAction = request.getParsedAction();
			if (parsedAction == null) {
				throw new IllegalArgumentException("No source action trying to get value for property: " + name);
			}
			return parsedAction.getPropertyAsObject(property);
		} else {
			return value;
		}
	}

	public void setName(String name) {
		this.name = name;
	}
	
	public void setValue(String value) {
		this.value = value;
	}

	public void setProperty(String property) {
		this.property = property;
	}
	
}
