/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.decorator;

import static org.loom.tags.AbstractFormInputTag.LabelPosition.LEFT;
import static org.loom.tags.AbstractFormInputTag.LabelPosition.RIGHT;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.i18n.Message;
import org.loom.tags.AbstractFormInputTag;
import org.loom.tags.WriterWrapper;
import org.loom.tags.messages.ErrorTag;

/**
 * Adds a label around the input field with its same css styles
 * @author icoloma
 */
public class AbstractFormInputDecorator<T extends AbstractFormInputTag> extends AbstractLoomDecorator<T> {

	public static enum ErrorPosition {
		
		/** render the error span as the last component inside the label */
		INSIDE,
		
		/** render the error span after the label */
		AFTER
		
	}
	
	/** where to render the error span */
	private ErrorPosition errorPosition = ErrorPosition.INSIDE;
	
	@Override
	public boolean beforeDoTag(T tag) throws IOException {
		if (tag.shouldRenderLabel()) {
			// do not add the translated name attribute
			tag.setAddTranslatedName(false);
			
			WriterWrapper<?> out = tag.getWriter();
			out.print("<label");
			out.printAttribute("class", tag.getLabelCssClass());
			out.print(">");
			
			if (LEFT.equals(tag.getLabelPosition())) {
				printLabel(out, tag, true);
			}
		}
		return true;
	}

	private void printLabel(WriterWrapper<?> wrapper, T tag, boolean printColon)
			throws IOException {
		wrapper.print("<span").printAttribute("class", "label").print(">");
		wrapper.print(tag.getTranslatedLabel());
		if (printColon){
			wrapper.print(":");
		}
		wrapper.print("</span>");
	}

	@Override
	public void afterDoTag(T tag) throws IOException, JspException {
		if (tag.shouldRenderLabel()) {
			if (RIGHT.equals(tag.getLabelPosition())) {
				printLabel(tag.getWriter(), tag, false);
			}
			if (errorPosition == ErrorPosition.INSIDE) {
				renderError(tag);
			}
			if (tag.getHelpLine() != null) {
				renderHelpLine(tag);
			}
			tag.getWriter().print("</label>\n");
		}
		if (errorPosition == ErrorPosition.AFTER) {
			renderError(tag);
		}
	}

	protected void renderError(T tag) throws JspException, IOException {
		if (tag.shouldRenderError()) {
			ErrorTag errorTag = new ErrorTag();
			errorTag.setJspContext(tag.getPageContext());
			errorTag.setName(tag.getName());
			errorTag.doTag();
		}
	}
	
	protected void renderHelpLine(T tag) throws IOException {
		String msg = tag.getMessagesRepository().translateMessage(new Message(tag.getHelpLine()));
		tag.getWriter().start("span", "class", "helpline").print(msg).end("span");
	}

	public void setErrorPosition(ErrorPosition errorPosition) {
		this.errorPosition = errorPosition;
	}

}
