/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.functions;

import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Random;

import javax.servlet.jsp.JspTagException;

import org.codehaus.jackson.map.ObjectMapper;
import org.loom.util.ClassUtils;
import org.loom.util.Lookup;
import org.loom.util.StringUtils;
import org.tldgen.annotations.Function;

/**
 * List of JSP functions included with Loom. For usage instructions, see loom-core.tld
 * @author icoloma
 */
public class Functions {
	
	private static Random random = new Random();
	
	/**
	 * Return a random number between 0 (inclusive) and the provided value (exclusive)
	 */
	@Function(example="${l:random(10)}")
	public static int random(int maxValue) {
		return random.nextInt(maxValue);
	}
	
	/**
	 * Return true if the collection/array contains the given item. If the collection is null, 
	 * returns false as the safe bet.
	 */
	@Function(example="&lt;c:if test=\"${l:contains(collection, element)}\"&gt;")
	public static boolean contains(Object collection, Object item ) throws JspTagException {
		if (collection == null) {
			return false;
		}
		if (collection.getClass().isArray()) {
			return Arrays.asList((Object[])collection).contains(item);
		} else if (collection instanceof Collection<?>) {
			return ((Collection<?>)collection).contains(item);
		} else {
			 throw new JspTagException("Invalid property exception. Expected java.util.Collection or object Array and found " + collection.getClass().getName());
		}
	}
	
    /**
     * Abbreviates a String using ellipsis. See 
     * <a href="http://commons.apache.org/lang/api/org/apache/commons/lang/StringUtils.html#abbreviate%28java.lang.String,%20int%29">StringUtils#abbreviate(String, int)</a> for details.
     */
	@Function(example="${l:truncate(veryLongStringVar, 8)}")
    public static String truncate(String str, int maxWidth) {
        return StringUtils.abbreviate(str, maxWidth);
    }
    
    /**
     * Serializes the provided argument as a JSON object
     */
	@Function(example="${l:json(value)}")
    public static String json(Object value) throws IOException {
    	if (value == null) {
			return "null";
		}
    	ObjectMapper objectMapper = Lookup.getInstance().byClass(ObjectMapper.class);
    	return objectMapper.writeValueAsString(value);
    }
	
	/**
	 * Serializes the provided argument as a JSON object using the provided view (see Jackson JsonView)
	 */
	@Function(example="${l:jsonView(value, 'com.acme.json.FullView')}")
	public static String jsonView(Object value, String viewClassName) throws IOException {
		if (value == null) {
			return "null";
		}
		Class<?> viewClass = ClassUtils.forName(viewClassName);
    	ObjectMapper objectMapper = Lookup.getInstance().byClass(ObjectMapper.class);
		return objectMapper.viewWriter(viewClass).writeValueAsString(value);
		
	}
}
