/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.menu;

import org.loom.action.Action;
import org.loom.json.JsonMarshaller;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.names.RequestAttributeNames;

/**
 * A pointer to the selected menu item.
 * 
 * Stored as a request attribute, indicates the menu item that should be marked as "selected". 
 * If absent, the selected item will be guessed.
 * @author icoloma
 */
public class SelectedMenuItemLocator {

	/** ID of the menu */
	private String menuId;
	
	/** ID of the selected menu item */
	private String menuItemId;
	
	/** action class of the selected menu item */
	private Class<? extends Action> actionClass;
	
	/** event of the selected menu item */
	private String event;

	public SelectedMenuItemLocator(String menuId, String menuItemId) {
		this.menuId = menuId;
		this.menuItemId = menuItemId;
	}
	
	public SelectedMenuItemLocator(String menuId, Class<? extends Action> actionClass, String event) {
		this.menuId = menuId;
		this.actionClass = actionClass;
		this.event = event;
	}
	
	/**
	 * get the currently selected menu item from the request object
	 */
	public static SelectedMenuItemLocator getSelectedMenuItem(LoomServletRequest request, String menuId) {
		return (SelectedMenuItemLocator) request.getAttribute(RequestAttributeNames.SELECTED_MENU_ITEM_PREFIX + menuId);
	}
	
	public static void setSelectedMenuItem(LoomServletRequest request, String menuId, Class<? extends Action> actionClass, String event) {
		SelectedMenuItemLocator locator = new SelectedMenuItemLocator(menuId, actionClass, event);
		request.setAttribute(RequestAttributeNames.SELECTED_MENU_ITEM_PREFIX + menuId, locator);
	}
	
	public static void setSelectedMenuItem(LoomServletRequest request, String menuId, String menuItemId) {
		SelectedMenuItemLocator locator = new SelectedMenuItemLocator(menuId, menuItemId);
		request.setAttribute(RequestAttributeNames.SELECTED_MENU_ITEM_PREFIX + menuId, locator);
	}
	
	public String getMenuId() {
		return menuId;
	}

	public String getMenuItemId() {
		return menuItemId;
	}

	public Class<? extends Action> getActionClass() {
		return actionClass;
	}

	public String getEvent() {
		return event;
	}
	
	@Override
	public String toString() {
		JsonMarshaller marshaller = new JsonMarshaller();
		marshaller.add("menuId", menuId);
		marshaller.add("menuItemId", menuItemId);
		marshaller.add("action", actionClass.getName());
		marshaller.add("event", event);
		return marshaller.toString();
	}
	
}
