/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.messages;

import java.io.IOException;
import java.util.Collection;

import javax.servlet.jsp.JspException;

import org.loom.i18n.Message;
import org.loom.i18n.MessageLevel;
import org.loom.i18n.Messages;
import org.loom.tags.AbstractTag;
import org.loom.util.StringUtils;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

import com.google.common.base.Predicate;
import com.google.common.collect.Collections2;
import com.google.common.collect.Lists;

/**
 * Prints unbound <code>Messages</code>. 
 * @see ErrorTag
 * @author icoloma
 */
@Tag(
		dynamicAttributes=true,
		bodyContent=BodyContent.EMPTY,
		example=
			"&lt;l:messages/>\n" +
			"&lt;l:messages properties=\"name,date,age\"/>\n" +
			"&lt;l:messages level=\"warn\"/>"
	)
public class MessagesTag extends AbstractTag {

	/** <a href="http://www.w3.org/TR/html4/struct/global.html#adef-id">HTML attribute</a> */
	@Attribute
	private String id;
	
	/** level of the messages to display, defaults to MessageLevel.ERROR */
	private MessageLevel level = MessageLevel.ERROR;
	
	/** the css class to use */
	private String cssClass;
	
	/** if set, display also messages bound to any of these properties */
	private String[] properties;
	
	/** if true, messages with smaller level of priority will also be displayed (otherwise only messages with strictly this level will be accepted). Defaults to false */
	private boolean lessOrEquals;

	@Override
	public void doTagImpl() throws JspException, IOException {

		if (id == null) {
			id = level == MessageLevel.ERROR? "errors" : level.name().toLowerCase();
		}
		
		// leave the div, even if empty, to enable javascript validation
		Collection<Message> filteredMessages = filterMessages();
		if (filteredMessages == null || filteredMessages.isEmpty()) {
			out.start("div", "id", id, "class", cssClass, "style", "display: none").end("div");
		} else {
			out.start("div", "id", id, "class", cssClass);
			out.print("<ul>");
			for (Message message : filteredMessages) {
				renderMessage(message);
			}
			out.print("</ul>");
			out.end("div");
			
		}
		
	}

	protected Collection<Message> filterMessages() {
		if (request.getParsedAction() == null) {
			return null;
		}
		Messages messages = request.getParsedAction().getMessages();
		Predicate<Message> predicate = new Predicate<Message>() {
			
			public boolean apply(Message candidate) {
				return (!lessOrEquals && candidate.getLevel() == level || lessOrEquals && candidate.getLevel().isLessOrEquals(level));
			}
			
		};
		Collection<Message> filteredMessages = Collections2.filter(messages.getUnboundMessages(), predicate);
		if (properties != null) {
			filteredMessages = Lists.newArrayList(filteredMessages);
			for (String property : properties) {
				Message candidate = messages.get(property);
				if (candidate != null && predicate.apply(candidate)) {
					filteredMessages.add(candidate);
				}
			}
		}
		return filteredMessages;
	}
	
	protected void renderMessage(Message message) throws IOException {
		out.start("li", "class", lessOrEquals? message.getLevel().getLowercaseName() : null);
		out.print(repository.translateMessage(message));
		out.print("</li>");
		message.setRendered(true);
	}
	
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * The level of messages to display. Accepts <code>info</code>, <code>warn</code> 
	 * and <code>error</code> (default)
	 */
	@Attribute
	public void setLevel(String level) {
		if (level.startsWith("<=")) {
			lessOrEquals = true;
			level = level.substring(2);
		}
		this.level = MessageLevel.lowerCaseValueOf(level);
	}

	/**
	 * <a href="http://www.w3.org/TR/html4/struct/global.html#adef-class">HTML attribute</a>
	 */
	@Attribute(name="class")
	public void setCssClass(String cssClass) {
		this.cssClass = cssClass;
	}
	
	/** 
	 * If set, display also messages bound to any of these properties.  
	 */
	@Attribute
	public void setProperties(String properties) {
		this.properties = StringUtils.split(properties, ", ");
	}
	
	public void setLessOrEquals(boolean includeSmallerLevels) {
		this.lessOrEquals = includeSmallerLevels;
	}
	
}
