/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.CharUtils;
import org.springframework.core.io.Resource;

public class StringUtils extends org.apache.commons.lang.StringUtils {
	
	/** characters to be replaced when generating identifiers */
	private static char[] REPLACE_ID_CHARS;
	
	/** 
	 * characters used to convert from number to string and viceversa 
	// 26 alpha lowercase characters  minus 'l'
	// 26 alpha uppercase characters minus 'I' and 'O'
	// 10 numeric digits minus '0'
	 * */
	private static final char[] BASE58_CHARS = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz".toCharArray();
	
	static {
		// see http://www.utf8-chartable.de/
		REPLACE_ID_CHARS = new char[256];
		addReplaceChars('\u00e7', 'c'); // ç
		addReplaceChars('\u00f1', 'n'); // ñ
		addReplaceChars('\u00e0', '\u00e6', 'a');
		addReplaceChars('\u00e8', '\u00eb', 'e');
		addReplaceChars('\u00ec', '\u00ef', 'i');
		addReplaceChars('\u00f2', '\u00f6', 'o');
		addReplaceChars('\u00f9', '\u00fc', 'u');
	}
	
	public static void addReplaceChars(char from, char to, char replacement) {
		for (char i = from; i <= to; i++) {
			REPLACE_ID_CHARS[i] = replacement;
		}
	}
	
	public static void addReplaceChars(char from, char replacement) {
		REPLACE_ID_CHARS[from] = replacement;
	}

	/**
	 * @return the camelcase concatenation of strings.
	 * Note: the first arg does not get uppercased. If needed,
	 * set the empty string as the first arg
	 */
	public static String camelCase(String... args) {
		if (args == null || args.length == 0) {
			return null;
		}
		StringBuilder buf = new StringBuilder();
		buf.append(args[0]);
		for (int i = 1; i < args.length; i++) {
			String arg = args[i];
			if (arg != null && arg.length() > 0) {
				buf.append(Character.toUpperCase(arg.charAt(0)));
				buf.append(arg.substring(1));
			}
		}
		return buf.toString();
	}
	
	/**
	 * Returns the camelCase result after using the provided separators array 
	 * to split the string into words.
	 * Example: camelCase("foo-bar.baz", "-.") = "fooBarBaz"
	 */
	public static String camelCaseWithSeparators(String text, String separators) {
		return camelCase(org.apache.commons.lang.StringUtils.split(text, separators));
	}
	
	/**
	 * Return true if the StringBuilder passed as parameter starts 
	 * with a known web prefix.
	 * This method will test for http://, https://, file:// and ://
	 */
	public static boolean hasProtocolPrefix(StringBuilder uri) {
		int length = uri.length();
		if (uri == null || length < 3) {
			return false;
		}
		
		int max = length;
		if (max > 10) { // maximum expected length of any protocol (guess value)
			max = 10;
		}
		
		for (int i = 0; i < max; i++) {
			char c = uri.charAt(i);
			switch (c) {
			case ':':
				return (i < length - 2 && uri.charAt(i + 1) == '/' && uri.charAt(i + 2) == '/') || // found '://'
					(i == 4 && uri.charAt(0) == 'f' && uri.charAt(1) == 'i' && uri.charAt(2) == 'l' && uri.charAt(3) == 'e'); // found 'file:'
					// else return false
			case '/': // found '/' without preceding ':', return false
				return false;
			default:
			}
		}
		
		return false;
		
	}

	/**
	 * Compares two Strings between the given chars
	 * @param s1
	 * @param s2
	 * @param from the start index, inclusive
	 * @param to the end index, exclusive
	 * @return true if are equals, false otherwise
	 */
	public static boolean compare(String s1, String s2, int from, int to) {
		for (int i = from; i < to; i++) {
			if (s1.charAt(i) != s2.charAt(i)) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * @return true if the String passed as argument contains at least one digit
	 */
	public static boolean hasDigits(String s) {
		int l = s.length();
		for (int i = 0; i < l; i++) {
			if (Character.isDigit(s.charAt(i))) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * @return the MD5 hash of the provided String
	 */
	public static String toMD5(String text) {
		try {
			return calculateMD5(text.getBytes("UTF-8"));
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Calculate the MD5 hash for the given byte array
	 * @param text the text to hash
	 * @return the corresponding MD5
	 */
	public static String calculateMD5(byte[] text) {
		
		try {
			MessageDigest md = MessageDigest.getInstance("MD5");
			md.update(text);
			return toString(md);
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		}

	}
	
	/**
	 * Transform a {@link MessageDigest} instance into its string representation
	 * @param md the message digest to process
	 * @return the string representation of this digest
	 */
	public static String toString(MessageDigest md) {
		byte[] digest = md.digest();
		StringBuilder md5 = new StringBuilder();
		for (int i = 0; i < digest.length; i++) {
			// this is done to keep leading zeroes of each byte
			String value = StringUtils.leftPad(Integer.toHexString(0xFF & digest[i]), 2, '0');
			md5.append(value);
		}
		return md5.toString();
	}
	
	/**
	 * Calculate the MD5 hash for the given resource
	 * @param resource the resource to hash
	 * @return the corresponding MD5
	 */
	public static String calculateMD5(Resource resource) throws IOException {
		// TODO use a buffer here
		InputStream input = resource.getInputStream();
		try {
			return StringUtils.calculateMD5(IOUtils.toByteArray(input));
		} finally {
			IOUtils.closeQuietly(input);
		}
	}
	
	/**
	 * Converts a camelCase string to a dash-separated one. Examples:
	 * myProperty ==> my-property
	 * MyProperty.something ==> my-property-something
	 * @param text the text to transform
	 * @return the dash-separated transformation of the provided text.
	 */
	public static String dasherize(String text) {
		return separate(text, '-');
	}
	
	/**
	 * Converts a camelCase string to a separated one. Receives the separator character
	 * @param text the text to transform
	 * @param separator the separator character to use
	 * @return the separated text.
	 */
	public static String separate(String text, char separator) {
		StringBuilder builder = new StringBuilder(text.length() + text.length() / 2);
		int l = text.length();
		for (int i = 0; i < l; i++) {
			char c = text.charAt(i);
			if (Character.isUpperCase(c)) {
				if (i != 0) {
					builder.append(separator);
				}
				builder.append(Character.toLowerCase(c));
			} else {
				builder.append(c);
			}
		}
		return builder.toString();
	}
	
	/**
	 * Transforms a text into an identifier candidate to be used in RESTful URLs.
	 * Spaces will be replaced by dashes, and everything that is not alphanumeric will be ignored
	 * Example: "My  2nd valentine (redux)" -> "my-second-valentine-redux"  
	 * @param text the text to transform
	 * @return the transformed text
	 * @throws IllegalArgumentException if the provided text cannot generate a valid identifier (that
	 * is, the generated identifier is empty or is '-')
	 */
	public static String generateIdentifier(String text) {
		text = text.trim();
		final int length = text.length();
		StringBuilder s = new StringBuilder(length);
		for (int i = 0; i < length; ) {
			char c = '\0';
			while (i < length) {
				c = Character.toLowerCase(text.charAt(i++));
				
				// known characters to replace
				if (!CharUtils.isAsciiAlphanumeric(c)) {
					char replace = c > REPLACE_ID_CHARS.length? '\0': REPLACE_ID_CHARS[c];
					if (replace == '\0') {
						break;
					} else {
						c = replace;
					}
				}
				s.append(c);
			}
			if (i < length) {
				s.append('-');
				while (i < length && !CharUtils.isAsciiAlphanumeric(text.charAt(i))) {
					i++;
				}
			}
			
		}
		if (s.length() == 0 || s.length() == 1 && s.charAt(0) == '-') {
			throw new IllegalArgumentException("Cannot generate an identifier for '" + text + "'");
		}
		return s.toString();
	}
	
	/**
	 * Convert a number into its (shorter) alphanumeric representation using Base58 chars ([0-9a-zA-Z] omitting 0, O, I and 1)  
	 * @param number the number to transform
	 * @return the text representation of this number
	 */
	public static String numberToAlpha(long number) {
		char[] buffer = new char[20];
		int index = 0;
		do {
			buffer[index++] = BASE58_CHARS[(int) (number % BASE58_CHARS.length)];
			number = number / BASE58_CHARS.length;
		} while (number > 0);
		return new String(buffer, 0, index);
	}
	
	/**
	 * Convert a short alphanumeric representation back to the original number using Base58
	 * @param text the alphanumeric representation
	 * @return the original number
	 */
	public static long alphaToNumber(String text) {
		char[] chars = text.toCharArray();
		long result = 0;
		long multiplier = 1;
		for (int index = 0; index < chars.length; index++) {
			char c = chars[index];
			int digit;
			if (c >= '1' && c <= '9') {
				digit = c - '1'; 
			} else if (c >= 'A' && c < 'I') {
				digit = (c - 'A') + 9;
			} else if (c > 'I' && c < 'O') {
				digit = (c - 'J') + 17;
			} else if (c > 'O' && c <= 'Z') {
				digit = (c - 'P') + 22;
			} else if (c >= 'a' && c < 'l') {
				digit = (c - 'a') + 33;
			} else if (c > 'l' && c <= 'z') {
				digit = (c - 'l') + 43;
			} else {
				throw new IllegalArgumentException("Illegal character found: '" + c + "'");
			}
				
			result += digit * multiplier;
			multiplier = multiplier * BASE58_CHARS.length;
		}
		return result;
	}
	
}
