/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.validator;

import java.math.BigDecimal;
import java.math.BigInteger;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.loom.tags.FormInputTag;
import org.loom.tags.core.AbstractTextFieldTag;
import org.loom.tags.core.NumberFieldTag;


/**
 * Class that contains data and behaviour to validate one field.
 * This class and its descendants are thread-safe.
 * @author Ignacio Coloma
 */
public class NumberValidator extends AbstractPropertyValidator {

	/** minimum allowed value, default 0 */
    private BigDecimal minValue;
    
    /** maximum allowed value */
    private BigDecimal maxValue;

    /** true to exclude the minimum, default false */
    private boolean excludeMin = false;
    
    /** true to exclude the maximum, default false */
    private boolean excludeMax = true;
    
    /** how many decimal digits @see javax.persistence.Column#precision()*/
	private Integer scale;
	
	/** how many digits, including decimal digits @see javax.persistence.Column#scale() */
	private Integer precision;
	
	/**
     * Validates one input value and adds to errors any validation error found 
     * @param v the value to validate
     */
    public void validate(ValidationRequest request, Object v) {
    	if (v != null) {
    		BigDecimal value = toBigDecimal((Number)v);
        	if (minValue != null) {
    			int compareResult = value.compareTo(minValue);
        		if ( (!excludeMin && compareResult < 0) || (excludeMin && compareResult <= 0) ) {
        			request.addErrorMessage("loom.validation.numberMinFailed", value);
        		}
        	}
        	if (maxValue != null) {
        		int compareResult = value.compareTo(maxValue);
        		if ( (!excludeMax && compareResult > 0) || (excludeMax && compareResult >= 0) ) {
        			request.addErrorMessage("loom.validation.numberMaxFailed", value);
        		}
        	}
        	if (scale != null && value.scale() > scale) {
        		request.addErrorMessage("loom.validation.scaleFailed", value);
        	}
        	if (precision != null && value.precision() > precision) {
        		request.addErrorMessage("loom.validation.precisionFailed", value);
        	}
    	}
    }
    
    private BigDecimal toBigDecimal(Number value) {
    	Class<?> type = value.getClass();
    	if (Integer.class.equals(type) || Integer.TYPE.equals(type)) {
			return new BigDecimal(value.intValue());
		}
    	if (Long.class.equals(type) || Long.TYPE.equals(type)) {
			return new BigDecimal(value.longValue());
		}
    	if (Double.class.equals(type) || Double.TYPE.equals(type)) {
			return new BigDecimal(value.toString());
		}
    	if (BigDecimal.class.equals(type)) {
			return (BigDecimal)value;
		}
    	// the rarest cases come now :)
    	if (Float.class.equals(type) || Float.TYPE.equals(type)) {
			return new BigDecimal(value.toString());
		}
    	if (Short.class.equals(type) || Short.TYPE.equals(type)) {
			return new BigDecimal(value.byteValue());
		}
    	if (Byte.class.equals(type) || Byte.TYPE.equals(type)) {
			return new BigDecimal(value.byteValue());
		}
    	if (BigInteger.class.equals(type)) {
			return new BigDecimal((BigInteger)value);
		}
    	throw new IllegalArgumentException("Don't know how to handle " + value + " of type: " + type.getName());
    }

    @Override
    public void decorate(FormInputTag t) {
    	super.decorate(t);
    	if (t instanceof AbstractTextFieldTag) {
    		AbstractTextFieldTag tag = (AbstractTextFieldTag) t;
    		if (tag.getMaxlength() == null) {
    			tag.setMaxlength(getMaxLength());
    		}
    	}
    	if (t instanceof NumberFieldTag) {
    		NumberFieldTag tag = (NumberFieldTag) t;
    		if (minValue != null) {
    			tag.setMin(minValue);
    			tag.setExcludeMin(excludeMin);
    		}
    		if (maxValue != null) {
    			tag.setMax(maxValue);
    			tag.setExcludeMax(excludeMax);
    		}
    		tag.setPrecision(precision);
    		tag.setScale(scale);
    	}
    }
	
	public boolean isExcludeMax() {
		return excludeMax;
	}

	public void setExcludeMax(boolean excludeMax) {
		this.excludeMax = excludeMax;
	}

	public boolean isExcludeMin() {
		return excludeMin;
	}

	public void setExcludeMin(boolean excludeMin) {
		this.excludeMin = excludeMin;
	}

	public Integer getPrecision() {
		return precision;
	}

	public Integer getScale() {
		return scale;
	}

	public void setPrecision(Integer precision) {
		this.precision = precision;
	}

	public void setScale(Integer scale) {
		this.scale = scale;
	}

	public void setMaxValue(BigDecimal maxValue) {
		this.maxValue = maxValue;
	}

	public void setMinValue(BigDecimal minValue) {
		this.minValue = minValue;
	}
	
    @Override
    public String toString() {
    	return new ToStringBuilder(this)
    		.append("propertyPath", getPropertyPath())
    		.append("minValue", minValue)
    		.append("maxValue", maxValue)
    		.append("excludeMin", excludeMin)
    		.append("excludeMax", excludeMax)
    		.append("scale", scale)
    		.append("precision", precision)
    		.toString();
    }

	public BigDecimal getMaxValue() {
		return maxValue;
	}

	public BigDecimal getMinValue() {
		return minValue;
	}

	/**
	 * Calculates the maxlength using the precision and max and min values specified
	 * @return null if it cannot be calculated, else the expected maxlength including possible 
	 * minus sign and decimal dot. 
	 */
	public Integer getMaxLength() {
		int result = precision == null? 0 : precision;
		if (result != 0 && scale != null && scale > 0) {
			result ++;
		}
		if (result == 0 && maxValue != null) { 
			result = getMaxLengthFor(result, minValue);
			result = getMaxLengthFor(result, maxValue);
		} 
		if (result != 0) { // possible minus sign
			if (minValue == null || minValue.compareTo(BigDecimal.ZERO) < 0) {
				result++;
			}
		}
		return result == 0? null : result;
	}

	/**
	 * Calculates the maxlength for the given number and if it's greater than the current maxLength, return that.
	 * Else, return maxLength.
	 */
	private Integer getMaxLengthFor(int currentMax, BigDecimal candidate) {
		if (candidate == null) {
			return currentMax;
		}
		int newLength = candidate.precision();
		if (candidate.scale() > 0) {
			newLength++;
		}
		return currentMax >= newLength? currentMax : newLength;
	}
	
}
