/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.validator;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.loom.i18n.Message;
import org.loom.tags.FormInputTag;
import org.loom.tags.core.AbstractTextFieldTag;


/**
 * Class that contains data and behaviour to validate one field.
 * This class and its descendants are thread-safe.
 * @author Ignacio Coloma
 */
public class StringValidator extends AbstractPropertyValidator {

	/** minimum length required, null if none (default null) */
    private Integer minLength;
    
    /** maximum length allowed, null if none (default null) */
    private Integer maxLength;
    
    /** the expected pattern */
    private Pattern maskPattern;
	
    @Override
    public void decorate(FormInputTag t) {
    	super.decorate(t);
    	if (t instanceof AbstractTextFieldTag) {
    		AbstractTextFieldTag tag = (AbstractTextFieldTag) t;
    		if (tag.getMaxlength() == null) {
    			tag.setMaxlength(maxLength);
    		}
    		tag.setMinlength(minLength);
    		tag.setPattern(maskPattern);
    	}

	}
	
	/**
     * Validates one input value and adds to errors any validation error found 
     * @param v the value to validate
     */
    public void validate(ValidationRequest request, Object v) {
    	String value = (String) v;
    	if (value != null) {
    		Message message = null;
    		if (minLength != null && value.length() < minLength.intValue()) {
    			message = request.addErrorMessage("loom.validation.minLengthFailed", value);
    		}
    		if (maxLength != null && value.length() > maxLength.intValue()) {
    			message = request.addErrorMessage("loom.validation.maxLengthFailed", value);
    		}
    		
    		// it may be null also if the field has associated Conversion errors
    		if (message !=  null) {
    			message.addArg("length", value.length());
    		}
    		
    		if (maskPattern != null) {
    			Matcher matcher = maskPattern.matcher(value);
    			if (!matcher.matches()) {
    				request.addErrorMessage("loom.validation.maskFailed", value);
    			}
    		}
    	}
    }

	public Pattern getMaskPattern() {
		return maskPattern;
	}

	public void setMaskPattern(Pattern mask) {
		this.maskPattern = mask;
	}

	public Integer getMaxLength() {
		return maxLength;
	}

	public void setMaxLength(Integer maxLength) {
		this.maxLength = maxLength;
	}

	public Integer getMinLength() {
		return minLength;
	}

	public void setMinLength(Integer minLength) {
		this.minLength = minLength;
	}

    @Override
    public String toString() {
    	return new ToStringBuilder(this)
    		.append("propertyPath", getPropertyPath())
    		.append("minLength", minLength)
    		.append("maxLength", maxLength)
    		.append("mask", maskPattern == null? null : maskPattern.pattern())
    		.toString();
    }
	
	
}
